from test_junkie.constants import TestCategory, Color, DocumentationLinks


class ReportTemplate:

    LABEL_COLOR = "#aebac4"

    @staticmethod
    def get_body_template():
        return """
                <!DOCTYPE html>
                <html lang="en">
                    <head>
                        <meta charset="UTF-8">
                        <title>Test Junkie HTML Report</title>
                        <script src="https://code.jquery.com/jquery-3.3.1.min.js"
                                integrity="sha256-FgpCb/KJQlLNfOu91ta32o/NMZxltwRo8QtmkMRdAu8="
                                crossorigin="anonymous"></script>
                        <script src="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.7.3/Chart.bundle.min.js"></script>
                        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/materialize/0.100.2/css/materialize.min.css">
                        <script src="https://cdnjs.cloudflare.com/ajax/libs/materialize/0.100.2/js/materialize.min.js"></script>
                        <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet">
                        <link href="https://cdn.datatables.net/1.10.19/css/jquery.dataTables.min.css" rel="stylesheet">
                        <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
                        <script src="https://www.amcharts.com/lib/4/core.js"></script>
                        <script src="https://www.amcharts.com/lib/4/charts.js"></script>
                        <script src="https://www.amcharts.com/lib/4/themes/dark.js"></script>
                        <script src="https://www.amcharts.com/lib/4/themes/animated.js"></script>
                        <script src="https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js"></script>
                    </head>
                    <body style="background-color: #3d4f58;">
                        <style>
                            .absolute_cards{{
                                min-height: 170px;
                                overflow: hidden;
                            }}

                            .absolute_value{{
                                font-size: 27px;
                            }}
                            
                            .modal_absolute_value{{
                                font-size: 17px;
                            }}

                            .card:hover {{
                                background: #193746 !important;
                                transition: all .3s ease-in;
                            }}
                            
                            body {{
                                font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
                            }}
                            
                            .success {{
                                background: #12d479;
                            }}
                            .fail {{
                                background: #fcd75f;
                            }}
                            .error {{
                                background: #ff7651;
                            }}
                            .ignore {{
                                background: #cce4eb;
                            }}
                            .skip {{
                                background: #34bff5;
                            }}
                            .cancel {{
                                background: #f19def;
                            }}
                            .info-badge {{
                                background: #aac7d2;
                            }}
                            
                            .parent-badge{{
                                margin-top: 8px;
                            }}
                            
                            .badge {{
                                border-radius: 3px;
                                margin-right: 7px;
                                color: #333 !important;
                            }}
                            
                            .traceback {{
                                background: #23323a !important;
                                padding: 15px !important;
                                color: #fd5858 !important;
                                margin: 0px !important;
                            }}
                            
                            .collapsible-header {{
                                background: #37474f !important;
                                border-color: #193746;
                            }}
                            
                            .collapsible.expandable {{
                                border: 1px #37474f;
                            }}
                            
                            .collapsible-header:hover {{
                                background: #193746 !important;
                                transition: all .3s ease-in;
                                transition: all .3s ease-out;
                            }}
                            
                            .collapsible-body {{
                                border-color: transparent;
                            }}
                            
                            .attempt-header {{
                                padding: 5px;
                                padding-bottom: 0px;
                                border: 0px;
                            }}
                            
                            .parameter {{
                              white-space: nowrap;
                              overflow: hidden;
                              text-overflow: ellipsis;
                              font-size: 14px;
                            }}
                            
                            html {{
                                overflow: scroll;
                                overflow-x: hidden;
                            }}
                            ::-webkit-scrollbar {{
                                width: 0px;  /* remove scrollbar space */
                                background: transparent;  /* optional: just make scrollbar invisible */
                            }}
                            /* optional: show position indicator in red */
                            ::-webkit-scrollbar-thumb {{
                                background: transparent;
                            }}
                            .traceback > textarea{{
                                padding-bottom: 0px;
                                margin-bottom: 0px;
                                overflow: auto;
                            }}
                            .large-data-card{{
                                width: 100%;
                                height: 370px;
                            }}
                        </style>
                        {body}
                        <script>
                            var database_lol = {database_lol}
                            function registerCopy(){{
                                $(".copy_icon").on("click", function(event){{
                                    event.stopPropagation();
                                    var copy_target = event.target.dataset.tcopy
                                    var data_element = document.getElementById(copy_target);
                                    if(data_element.tagName == "textarea") var val = data_element.value
                                    else var val = data_element.innerHTML
                                    
                                    var $temp = $("<input>");
                                    $("body").append($temp);
                                    $temp.val(val).select();
                                    document.execCommand("copy");
                                    $temp.remove();
                                    
                                    Materialize.toast('Copied!', 2000)
                                }});
                            }};
                        </script>
                    </body>
                </html> 
               """

    @staticmethod
    def get_donation_options():

        return """
                <div class="fixed-action-btn horizontal">
                    <a class="btn-floating btn-large waves-effect waves-light orange darken-4 modal-trigger" 
                    href="#developerModal"><i class="material-icons">favorite</i></a>
                </div>
                <script>
                    $(document).ready(function(){
                        $('#developerModal').modal();
                    });
                </script>
                <div id="developerModal" class="modal">
                    <div class="modal-content" style="color: black;">
                      <h4>Greetings!</h4>
                        <p>Some features are not yet finished. 
                           I'm <a rel="noopener" href="https://www.linkedin.com/in/arturspirin/" target="_blank">
                           Artur <i class='fas fa-external-link-alt'></i></a> by the way, the developer of
                           <a href="https://www.test-junkie.com/" target="_blank" rel="noopener">Test Junkie 
                           <i class='fas fa-external-link-alt'></i></a></p>
                        <p>It took a lot of effort, all my free time outside my day job, and many sleepless
                            nights to get this project to this point, that includes styling and doing data
                            processing in order to render all those charts and tables. I have been motivated by
                            the idea that it may become useful for testers, QA managers, QA team leads, and
                            even project managers and developers. Thus I open sourced it.</p>
                        <p>Ask yourself if all the effort I put in and the benefits Test Junkie brings to you
                            and/or your company (if any) are worth a donation or 2 minutes of your time.</p>
                        <p>Ways you can help:</p>
                        <ol>
                            <li>The biggest thing you can do for me is 
                                <a class="github-button" href="https://github.com/ArturSpirin/test_junkie" 
                                data-size="large" data-show-count="true" aria-label="Star ArturSpirin/test_junkie on 
                                GitHub">Star</a> the project on GitHub and share it on social media, with your friend 
                                and/or colleagues.</li>
                            <li>Write me a message on <a rel="noopener" href="https://www.linkedin.com/in/arturspirin/"
                                target="_blank">Linked In <i class='fas fa-external-link-alt'></i></a> and tell me 
                                about your experience with Test Junkie.</li>
                            <li>If you can afford, you can donate through
                                <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=FJPYWX5B776YS
                                &currency_code=USD&source=url" target="_blank" rel="noopener">PayPal 
                                <i class='fas fa-external-link-alt'></i></a>. Every penny donated motivates me more and 
                                more to spend time on this project.</li>
                            <li>You can also back me on
                                <a rel="noopener" href="https://www.patreon.com/join/arturspirin/" target="_blank">
                                Patreon <i class='fas fa-external-link-alt'></i></a> on behalf of your company and 
                                get some benefits by doing so.</li>
                        </ol>
                        <p>Happy testing!</p>
                    </div>
                    <div class="modal-footer">
                      <a href="#!" class="modal-close waves-effect waves-green btn-flat">Close</a>
                    </div>
                    <script async defer src="https://buttons.github.io/buttons.js"></script>
                </div>
               """

    @staticmethod
    def get_tiny_card_template(label, data, tooltip=None):
        return """
                    <div class="col s6 m4 l3 xl2">
                      <div class="card blue-grey darken-3 absolute_cards {tooltipped}" {tooltip}>
                        <div class="card-content white-text">
                          <span class="card-title">{label}</span>
                          <div class="absolute_value"><b>{data}</b></div>
                        </div>
                      </div>
                    </div>
                   """.format(label=label, data=data,
                              tooltip="data-html='true' data-tooltip='{}'".format(tooltip) if tooltip else "",
                              tooltipped="tooltipped"if tooltip else "")

    @staticmethod
    def get_table(data):
        def css():
            return """
                   <style>
                        table.dataTable thead th, table.dataTable thead td, table.dataTable.no-footer{
                        border: 0px !important;}
                        table.dataTable.no-footer{padding-bottom: 10px;}
                        #table_results tr{background: transparent;}
                        #table_results tbody tr:hover{
                            background: #1e4254;
                            cursor: pointer;
                            border-radius: 3px;}
                        #table_results_info{color: #ffffff;}
                        .dataTables_wrapper .dataTables_paginate{padding-right:50px;}
                        .dataTables_wrapper .dataTables_paginate .paginate_button.current, .dataTables_wrapper .dataTables_paginate .paginate_button {
                            color: #ffffff !important;
                            background: transparent !important;
                            border: 1px transparent!important;}
                        .dataTables_wrapper .dataTables_paginate .paginate_button.current, .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover {
                            color: #ffffff !important;
                            border-bottom: 2px #fff !important;}
                        .dataTables_wrapper .dataTables_paginate .paginate_button:hover {
                            background: transparent !important;
                            border: 1px transparent!important;}
                        .dataTables_wrapper .dataTables_paginate .paginate_button:active {
                            background: transparent !important;
                            border: 1px transparent!important;}
                        .dataTables_wrapper .dataTables_paginate .paginate_button.current{
                            background: #1e4254 !important;}
                        .trace-label {
                            width: 120px;
                            background: #23323a;
                            color: #cac531;
                            position: relative;
                            padding: 3px;
                            padding-left: 8px;}
                        .trace-label:before {
                            content: '';
                            position: absolute;
                            top: 0; right: 0;
                            border-top: 28px solid #3c4f58;
                            border-left: 28px solid #23323a;
                            width: 0;}
                        .runtime-label {
                            width: 120px;
                            background: #23323a;
                            position: relative;
                            padding: 3px;
                            margin-bottom: 15px;
                            padding-right: 8px;}
                        .runtime-label:before {
                            content: '';
                            position: absolute;
                            top: 0; left: 0;
                            border-bottom: 28px solid #3c4f58;
                            border-right: 28px solid #23323a;
                            width: 0;}
                   </style>
                   """

        def js():
            return """
                   <script>
                        $(document).ready(function() {

                            var table = $("#table_results").DataTable( {
                                deferRender: true,
                                data: {data},
                                displayLength: 25,
                                order: [[ 0, "asc" ]],
                                columns: [
                                    {
                                        title: "Suite Name" ,
                                        mDataProp: "suite",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Test Name" ,
                                        mDataProp: "test",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Feature" ,
                                        mDataProp: "feature",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Component" ,
                                        mDataProp: "component",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Assignee" ,
                                        mDataProp: "assignee",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Duration" ,
                                        mDataProp: "duration",
                                        defaultContent: "N/A"},
                                    {
                                        title: "Status" ,
                                        mDataProp: "status",
                                        defaultContent: "N/A"}],
                                createdRow: function(row, data, index){
                                    row.setAttribute("href", "#testCaseModal")
                                    row.setAttribute('class', 'modal-trigger')
                                    row.setAttribute('data-test_id', data.test_id)
                                    row.setAttribute('data-suite_id', data.suite_id)
                                }
                            });

                            var example_length = $('#table_results_length');
                            var example_filter = $('#table_results_filter');
                            example_filter.html("")
                            example_length.html("")

                            $('#table_results thead th').each(function(){
                                var title = $('#table_results thead th').eq($(this).index()).text();
                                $(this).html('<input class="deletable" id="search'+$('#table_results thead th').eq($(this).index()).text()+'" type="text" placeholder="'+title+'"/>');
                            });

                            table.columns().every(function(){
                                var tableColumn = this;
                                var input = $(this.header()).find('input');

                                input.on('change', function(){
                                    tableColumn.search(this.value).draw();
                                });
                                input.on('click', function(e){
                                    e.stopPropagation();
                                });
                                input.keyup(function(e){
                                    tableColumn.search(this.value).draw();
                                    input.focus();
                                });
                                input.keydown(function(e){
                                    if (e.keyCode == 65 && e.ctrlKey) {
                                        e.target.select()
                                    }
                                });
                            });
                        });
                   </script>
                   """.replace("{data}", str(data))

        return """
                <div class="row">
                    <div class="col s12 m12 l12 xl12">
                      <div class="card blue-grey darken-3">
                            <div class="card-content white-text">
                                <table id="table_results" class="mdl-data-table" ></table>
                                {css}
                                {js}
                            </div>
                        </div>
                    </div>
                    
                    <div id="testCaseModal" class="modal" style="background-color: #3d4f58; color: #bbc3c7">
                        <div id="testCaseModalContent" class="modal-content">
                              <div class="row"><span style="font-size: 22px;" id="test_name"></span></div>
                              <div class="row" id="suite_metrics"></div>
                              <div class="row"><ul class="collapsible expandable" id="test_details"></ul></div>
                        </div>
                    </div>
                    <script>
                        $(document).ready(function(){{
                            $('#testCaseModal').modal();
                            var test_modal = $("#testCaseModal")
                            
                            function register_tr_clicks(){{
                                $("tr.modal-trigger").on("click", function(event){{
                                    var row = $(this)[0].dataset
                                    var test_id = row.test_id
                                    var suite_id = row.suite_id
                                    render_modal(test_id, suite_id)
                                }})
                            }}
                            
                            register_tr_clicks()
                            $("#table_results").on( 'draw.dt', function () {{
                                console.log("table redrawn")
                                register_tr_clicks()
                            }});

                            function render_modal(test_id, suite_id){{
                                var status = database_lol.tests[test_id]["status"]
                                var name = database_lol.tests[test_id]["name"]
                                var suite = database_lol.suites[suite_id]["name"]
                                var module = database_lol.suites[suite_id]["module"]
                                
                                $("#test_name").html("<span data-tooltip="+module+"."+suite+"."+name+"() class='tooltipped'><i class='material-icons'>near_me</i> <b>"+name+"()</b></span>")
                                
                                var suite_metrics_html = ""
                                var suite_metrics = database_lol.suites[suite_id].metrics
                                for(index in suite_metrics){{
                                    var data = suite_metrics[index]
                                    suite_metrics_html += '<div class="col s12 m6 l6 xl3">'                                
                                    suite_metrics_html += '<div class="card blue-grey darken-3 absolute_cards">'                                
                                    suite_metrics_html += '<div data-position="bottom" data-tooltip="This metrics show data for the entire suite: '+suite+' and not just for this test case" class="tooltipped card-content" style="color: #bbc3c7;">'
                                    suite_metrics_html += '<span>Performance</span>'                                
                                    suite_metrics_html += '<span class="card-title">@'+index+'</span>'                                
                                    suite_metrics_html += '<div class="modal_absolute_value">avg <b class="right">'+data.avg+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">median <b class="right">'+data.median+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">min <b class="right">'+data.minimum+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">max <b class="right">'+data.maximum+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">total <b class="right">'+data.total+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">executions <b class="right">'+data.executions+'</b></div>'
                                    suite_metrics_html += '<div class="modal_absolute_value">failures <b class="right">'+data.failures+'</b></div>'
                                    suite_metrics_html += '</div></div></div>'                                
                                }}
                                $("#suite_metrics").html(suite_metrics_html)
                                
                                var inject_html = ""
                                var data = database_lol.tests[test_id].metrics
                                var distinct_index = 0
                                
                                for(class_param in data){{
                                    var class_data = data[class_param]
                                    distinct_index += 1
                                    for(test_param in class_data){{
                                        distinct_index += 1
                                        
                                        var test_data = class_data[test_param]
                                        
                                        var actual_suite_param = test_data.class_param
                                        if(actual_suite_param == null) actual_suite_param = "N/A"
                                        else actual_suite_param = class_param
                                        
                                        var actual_test_param = test_data.param
                                        if(actual_test_param == null) actual_test_param = "N/A"
                                        else actual_test_param = test_param
                                        
                                        var status = test_data.status
                                        var retry = test_data.retry
                                        var params_total = test_data.params_total
                            
                                        var params_ul = '<li class="bg">'
                            
                                        params_ul += '<div class="collapsible-header bg">'
                                            params_ul += '<div class="col xl1 badge_value"><span class="badge '+status+'">'+status+'</span></div>'
                                            params_ul += '<div class="col xl1 badge_value"><span class="badge info-badge">'+params_total+'</span></div>'
                                            params_ul += '<div class="col xl10">'
                                                params_ul += '<div data-position="bottom" class="col xl6 parameter"><a data-tooltip="Copy class parameters" class="tooltipped waves-effect copy_icon"><i data-tcopy="suite_param_'+distinct_index+'" class="material-icons tiny right">content_copy</i></a>Class parameter: <b><span id="suite_param_'+distinct_index+'" class="tooltipped" data-tooltip="'+actual_suite_param+'">'+actual_suite_param+'</span></b></div>'
                                                params_ul += '<div data-position="bottom" class="col xl6 parameter"><a data-tooltip="Copy test parameters" class="tooltipped waves-effect copy_icon"><i data-tcopy="test_param_'+distinct_index+'" class="material-icons tiny right">content_copy</i></a>Test parameter: <b><span id="test_param_'+distinct_index+'" class="tooltipped" data-tooltip="'+actual_test_param+'">'+actual_test_param+'</span></b></div>'
                                            params_ul += '</div>'
                                        params_ul += '</div>'    
                                        
                                        params_ul += '<div class="collapsible-body">'
                                        
                                        var details_ul = '<ul class="collapsible expandable">'
                                        for(index in test_data.performance){{
                                            distinct_index += 1
                                            var attempt = parseInt(index) + 1
                                            
                                            var beforeTestTraceback = test_data.beforeTest.tracebacks[index]
                                            var afterTestTraceback = test_data.afterTest.tracebacks[index]
                                            var testTraceback = test_data.tracebacks[index]
                                            
                                            var beforeTestDuration = test_data.beforeTest.performance[index]
                                            var testDuration = test_data.performance[index]
                                            var afterTestDuration = test_data.afterTest.performance[index]
                                            
                                            var beforeTestStatus = "N/A"
                                            if(beforeTestTraceback == null && beforeTestDuration != null) beforeTestStatus = "OK"
                                            else if(beforeTestTraceback){{
                                                if(beforeTestTraceback.includes("AssertionError")) beforeTestStatus = "Fail"
                                                else beforeTestStatus = "Error"
                                            }}

                                            var afterTestStatus = "N/A"
                                            if(afterTestTraceback == null && afterTestDuration != null){{
                                                if (["N/A", "OK"].includes(beforeTestStatus)) afterTestStatus = "OK"
                                            }}
                                            else if(afterTestTraceback){{
                                                if(afterTestTraceback != afterTestStatus){{
                                                    if(afterTestTraceback.includes("AssertionError")) afterTestStatus = "Fail"
                                                    else afterTestStatus = "Error"
                                                }}
                                            }}
                                            
                                             var new_li = '<li>'
                                                new_li += '<div class="collapsible-header attempt-header">'
                                                    new_li += '<div class="col s1 m1 l1 xl1">'
                                                        new_li += '<i class="material-icons">fingerprint</i>'
                                                    new_li += '</div>'
                                                    new_li += '<div class="col s11 m11 l11 xl11">'
                                                        new_li += '<div class="col s3 m3 l3 xl3">'
                                                            new_li += '<span>Attempt: <span class="badge info-badge">'+attempt+'</span></span>'
                                                        new_li += '</div>'
                                                        new_li += '<div class="col s3 m3 l3 xl3">'
                                                            new_li += '<span>Runtime: <span data-position="bottom" class="badge info-badge">'+testDuration+'</span></span>'
                                                        new_li += '</div>'
                                                        new_li += '<div class="col s3 m3 l3 xl3">'
                                                            new_li += '<span>@beforeTest: <span class="badge info-badge">'+beforeTestStatus+'</span></span>'
                                                        new_li += '</div>'
                                                        new_li += '<div class="col s3 m3 l3 xl3">'
                                                            new_li += '<span>@afterTest: <span class="badge info-badge">'+afterTestStatus+'</span></span>'
                                                        new_li += '</div>'
                                                    new_li += '</div>'
                                                new_li += '</div>'
                                                
                                                if(beforeTestTraceback == null) beforeTestTraceback = "OK"
                                                if(testTraceback == null) testTraceback = "OK"
                                                if(afterTestTraceback == null) afterTestTraceback = "OK"
                                                new_li += '<div class="collapsible-body">'
                                                    var height = 150
                                                    var color = '#fd5858'
                                                    if(beforeTestTraceback == 'OK'){{
                                                        height = 40
                                                        color = '#10d478' 
                                                    }}
                                                    if(beforeTestDuration != null){{
                                                        new_li += '<div class="trace-label"><span>@beforeTest</span></div>'
                                                        new_li += '<div class="traceback"><textarea disabled style="overflow: auto; color: '+color+'; height: '+height+'px;" class="materialize-textarea">'+beforeTestTraceback+'</textarea></div>'
                                                        new_li += '<div class="right runtime-label"><span class="right">'+beforeTestDuration+'</span></div>'
                                                        new_li += '<br>'
                                                    }}
                                                    height = 150
                                                    color = '#fd5858'
                                                    if(testTraceback == 'OK'){{
                                                        height = 40
                                                        color = '#10d478' 
                                                    }}
                                                    new_li += '<div class="trace-label"><span>@test</span></div>'
                                                    new_li += '<div class="traceback"><textarea id="trace'+distinct_index+'" disabled style="overflow: auto; color: '+color+'; height: '+height+'px;" class="materialize-textarea">'+testTraceback+'</textarea></div>'
                                                    new_li += '<div class="right runtime-label"><span class="right">'+testDuration+'</span></div>'
                                                    new_li += '<br>'
                                                    height = 150
                                                    color = '#fd5858'
                                                    if(afterTestTraceback == 'OK'){{
                                                        height = 40
                                                        color = '#10d478' 
                                                    }}
                                                    if(afterTestDuration != null){{
                                                        new_li += '<div class="trace-label"><span>@afterTest</span></div>'
                                                        new_li += '<div class="traceback"><textarea disabled style="overflow: auto; color: '+color+'; height: '+height+'px;" class="materialize-textarea">'+afterTestTraceback+'</textarea></div>'
                                                        new_li += '<div class="right runtime-label"><span class="right">'+afterTestDuration+'</span></div>'
                                                        new_li += '<br>'
                                                    }}
                                                new_li += '</div>'
                                             new_li += '</li>'
                                             details_ul += new_li
                                        }}
                                        details_ul += '</ul>'
                                        params_ul += details_ul
                                        params_ul += '</div>'
                                        params_ul += '</li>'
                                        inject_html += params_ul
                                    }}
                                }}
                                $("#test_details").html(inject_html)
                                $('.collapsible.expandable').collapsible();
                                $('.tooltipped').tooltip();
                                registerCopy();
                            }}

                        }});
                    </script>
                </div>
               """.format(css=css(), js=js())

    @staticmethod
    def get_resource_chart_template(data):
        """
        :param data: LIST of DICTs aka
                     [{date: newDate, cpu: cpu, mem: mem},
                      {date: newDate, cpu: cpu, mem: mem}]
        :return: STRING, HTML for the CPU & MEM chart
        """
        doc_link = "<span>Resource monitoring disabled. See <a href=\"{link}\">documentation " \
                   "<i class='fas fa-external-link-alt'></i></a> to enabled it.</span>"\
                   .format(link=DocumentationLinks.HTML_REPORT)
        data = [] if data is None else data

        def css():

            return """
                   <style>
                        #resources {
                            width: 100%;
                            height: 282px;
                        }
                   </style>
                   """

        def js():

            return """
                   <script>
                   $(document).ready(function() {
                        // Themes
                        am4core.useTheme(am4themes_dark);
                        am4core.useTheme(am4themes_animated);
                        
                        // Create chart
                        var chart = am4core.create("resources", am4charts.XYChart);
                        chart.paddingRight = 20;
                        
                        // format data
                        var data = {data};
                        for(dict in data) data[dict].date = new Date(data[dict].date)
                        
                        chart.data = data;
    
                        var dateAxis = chart.xAxes.push(new am4charts.DateAxis());
                        dateAxis.baseInterval = {
                          "timeUnit": "second",
                          "count": 1
                        };
                        dateAxis.tooltipDateFormat = "HH:mm:ss, d MMMM";
    
                        var valueAxis = chart.yAxes.push(new am4charts.ValueAxis());
                        valueAxis.tooltip.disabled = true;
                        valueAxis.title.text = "Usage %";
                        valueAxis.strictMinMax  = 100;
    
                        function createSeries(field, name){
                          var series = chart.series.push(new am4charts.LineSeries());
                          series.name = name;
                          series.dataFields.dateX = "date";
                          series.dataFields.valueY = field;
                          series.tooltipText = name+": [bold]{valueY}%[/]";
                          series.fillOpacity = 0.3;
                          return series
                        }
    
                        createSeries("cpu", "cpu")
                        createSeries("mem", "mem")
                        chart.cursor = new am4charts.XYCursor();
                        chart.cursor.lineY.opacity = 0;
                    });
                   </script>
                   """.replace("{data}", str(data))

        return """
                <div class='col s12 m12 l6 xl6'>
                  <div class='card blue-grey darken-3'>
                    <div class='large-data-card card-content white-text'>
                      <span class='card-title'>CPU & Memory Usage:</span>
                      {doc_link}
                      <div id="resources"></div>
                      {css}
                      {js}
                    </div>
                  </div>
                </div>
               """.format(css=css(), js=js(), doc_link=doc_link if not data else "")

    @staticmethod
    def get_absolute_results_template(data, colors):
        """
        Colors need to come with the data as well
        :param data: LIST of DICTs aka
                     [{status: "Pass", value: 401},
                      {status: "Fail", value: 300},
                      {status: "Error", value: 200},
                      {status: "Ignore", value: 165},
                      {status: "Canceled", value: 139},
                      {status: "Skipped", value: 128}]
        :return: STRING, HTML for the Absolute Metrics chart
        """
        def css():

            return """
                   <style>
                       #absolute_metrics{
                            width: 100%;
                            height: 282px;
                       }
                   </style>
                   """

        def js():

            return """
                   <script>
                        // Theme
                        am4core.useTheme(am4themes_dark);
                        am4core.useTheme(am4themes_animated);

                        var chart = am4core.create("absolute_metrics", am4charts.PieChart);
                        chart.hiddenState.properties.opacity = 0; // this creates initial fade-in

                        chart.data = {data};
                        chart.radius = am4core.percent(70);
                        chart.innerRadius = am4core.percent(40);
                        chart.startAngle = 180;
                        chart.endAngle = 360;

                        var series = chart.series.push(new am4charts.PieSeries());
                        
                        // format colors
                        var colors = {colors}
                        for (item in colors) colors[item] = am4core.color(colors[item]) 
                        series.colors.list = colors;
                        
                        series.dataFields.value = "value";
                        series.dataFields.category = "status";
                        series.slices.template.cornerRadius = 10;
                        series.slices.template.innerCornerRadius = 7;
                        series.slices.template.draggable = true;
                        series.slices.template.inert = true;
                        series.alignLabels = false;
                        series.hiddenState.properties.startAngle = 90;
                        series.hiddenState.properties.endAngle = 90;
                   </script>
                   """.replace("{data}", str(data)).replace("{colors}", str(colors))

        return """
                <div class="col s12 m12 l6 xl6">
                  <div class="card blue-grey darken-3">
                    <div class="large-data-card card-content white-text">
                      <span class="card-title">Absolute Metrics:</span>
                        <div id="absolute_metrics"></div>
                        {css}
                        {js}
                    </div>
                  </div>
                </div>
               """.format(css=css(), js=js())

    @staticmethod
    def get_stacked_bar_results_template(features_data, components_data, team_data, suites_data, tags_data):
        """
        All data must be passed in in the following format:
        [{"duration": 10,
          "measure": "Login",
          "passed": 227,
          "error": 10,
          "cancel": 12,
          "ignore": 40,
          "failed": 408
        }, {"duration": 10,
            "measure": "Dashboard",
            "passed": 371,
            "error": 10,
            "cancel": 7,
            "ignore": 38,
            "failed": 482}, ...]
        :param features_data: LIST of DICTs
        :param components_data: LIST of DICTs
        :param team_data: LIST of DICTs
        :param suites_data: LIST of DICTs
        :param tags_data: LIST of DICTs
        :return: STRING, HTML for all the charts in the tabs
        """

        series = []
        for status in TestCategory.ALL:
            series.append({"status": status, "label": status, "color": Color.MAPPING[status]})

        def css(chart_id):

            return """
                    <style>
                        #{chart_id} {{
                            width: 100%;
                            height: 400px;
                        }}
                    </style>
                   """.format(chart_id=chart_id)

        def js(chart_id, chart_data):

            return """
                   <script>
                   $(document).ready(function() {
                        // Themes begin
                        am4core.useTheme(am4themes_animated);
                        am4core.useTheme(am4themes_dark);
                        // Create chart instance
                        var chart_{chart_id} = am4core.create("{chart_id}", am4charts.XYChart);
                        
                        // Add data
                        chart_{chart_id}.data = {data}
                        
                        // Create axes
                        var categoryAxis_{chart_id} = chart_{chart_id}.xAxes.push(new am4charts.CategoryAxis());
                        categoryAxis_{chart_id}.dataFields.category = "measure";
                        categoryAxis_{chart_id}.renderer.grid.template.location = 0;
                        categoryAxis_{chart_id}.renderer.labels.template.wrap = true;
                        categoryAxis_{chart_id}.renderer.labels.template.maxWidth = 120;
                        
                        var valueAxis_{chart_id} = chart_{chart_id}.yAxes.push(new am4charts.ValueAxis());
                        valueAxis_{chart_id}.renderer.inside = true;
                        valueAxis_{chart_id}.renderer.labels.template.disabled = true;
                        valueAxis_{chart_id}.min = 0;
                        
                        var testsAxis_{chart_id} = chart_{chart_id}.yAxes.push(new am4charts.ValueAxis());
                        testsAxis_{chart_id}.title.text = "Tests";
                        testsAxis_{chart_id}.renderer.grid.template.disabled = true;
                        
                        var durationAxis_{chart_id} = chart_{chart_id}.yAxes.push(new am4charts.DurationAxis());
                        durationAxis_{chart_id}.title.text = "Average Test Duration";
                        durationAxis_{chart_id}.baseUnit = "second";
                        durationAxis_{chart_id}.renderer.grid.template.disabled = true;
                        durationAxis_{chart_id}.renderer.opposite = true;
                        durationAxis_{chart_id}.durationFormatter.durationFormat = "h'h' m'min' s'sec'";
                        
                        // Create series
                        function createSeries(field, name, color) {
                              // Set up series
                              var series_{chart_id} = chart_{chart_id}.series.push(new am4charts.ColumnSeries());
                              series_{chart_id}.name = name;
                              series_{chart_id}.yAxis = testsAxis_{chart_id};
                              series_{chart_id}.dataFields.valueY = field;
                              series_{chart_id}.dataFields.categoryX = "measure";
                              series_{chart_id}.sequencedInterpolation = true;
                              series_{chart_id}.legendSettings.valueText = "{valueY}"
                            
                              // Make it stacked
                              series_{chart_id}.stacked = true;
                            
                              // Configure columns
                              series_{chart_id}.columns.template.width = am4core.percent(60);
                              series_{chart_id}.columns.template.tooltipText = "[bold]{name}[/]: {valueY}";
                              series_{chart_id}.columns.template.stroke = am4core.color(color);
                              series_{chart_id}.columns.template.fill = am4core.color(color);
                              series_{chart_id}.columns.template.fillOpacity = 0.9;
                              return series_{chart_id};
                        }
                        
                        var required_series_{chart_id} = {series}
                        for(entry in required_series_{chart_id}){
                            createSeries(required_series_{chart_id}[entry].status, required_series_{chart_id}[entry].label, required_series_{chart_id}[entry].color)
                        }
                        
                        var durationSeries_{chart_id} = chart_{chart_id}.series.push(new am4charts.LineSeries());
                        durationSeries_{chart_id}.dataFields.valueY = "duration";
                        durationSeries_{chart_id}.dataFields.categoryX = "measure";
                        durationSeries_{chart_id}.yAxis = durationAxis_{chart_id};
                        durationSeries_{chart_id}.name = "Duration";
                        durationSeries_{chart_id}.strokeWidth = 2;
                        durationSeries_{chart_id}.stroke = am4core.color("#03a2cd");
                        durationSeries_{chart_id}.tooltip.getFillFromObject = false;
                        durationSeries_{chart_id}.tooltip.background.fill = am4core.color("#03a2cd");
                        durationSeries_{chart_id}.tooltipText = "{valueY.formatDuration()}";
                        durationSeries_{chart_id}.legendSettings.valueText = "{valueY}"
                        
                        var bullet_{chart_id} = durationSeries_{chart_id}.bullets.push(new am4charts.Bullet());
                        var circle_{chart_id} = bullet_{chart_id}.createChild(am4core.Circle);
                        circle_{chart_id}.radius = 4;
                        circle_{chart_id}.fill = am4core.color("#fff");
                        circle_{chart_id}.strokeWidth = 3;
                        
                        // Legend
                        chart_{chart_id}.legend = new am4charts.Legend();
                        
                        // Add cursor
                        chart_{chart_id}.cursor = new am4charts.XYCursor();
                    });
                   </script>
                   """.replace("{data}", str(chart_data))\
                      .replace("{chart_id}", chart_id)\
                      .replace("{series}", str(series))

        def card(chart_id, title, chart_data):
            return """
                   <div id="{chart_id}_tab" class="col s12">
                       <div class="col s12 m12 l12 xl12">
                         <div class="card blue-grey darken-3">
                           <div class="card-content white-text">
                             <span class="card-title">{title}:</span>
                             <div id="{chart_id}"></div>
                             {css}
                             {js}
                           </div>
                         </div>
                       </div>
                   </div>
                   """.format(chart_id=chart_id, title=title, css=css(chart_id), js=js(chart_id, chart_data))

        features_html = card("features", "Results by Features", features_data)
        components_html = card("components", "Results by Components", components_data)
        team_html = card("owners", "Results by Assignees", team_data)
        suites_html = card("suites", "Results by Test Suites", suites_data)
        tags_html = card("tags", "Results by Tags", tags_data)

        return """
                <div class="row">
                    <div class="col s12 m12 l12 xl12">
                      <ul class="tabs transparent" style="margin-left: 20px;">
                        <li class="tab col s2"><a class="blue-text text-lighten-2 active" href="#features_tab">Features</a></li>
                        <li class="tab col s2"><a class="blue-text text-lighten-2" href="#components_tab">Components</a></li>
                        <li class="tab col s2"><a class="blue-text text-lighten-2" href="#owners_tab">Team</a></li>
                        <li class="tab col s2"><a class="blue-text text-lighten-2" href="#suites_tab">Suites</a></li>
                        <li class="tab col s2"><a class="blue-text text-lighten-2" href="#tags_tab">Tags</a></li>
                        <div class="indicator blue" style="z-index:1"></div>
                      </ul>
                    </div>
                    {features}
                    {components}
                    {team}
                    {suites}
                    {tags}
                </div>
               """.format(features=features_html, components=components_html, team=team_html,
                          suites=suites_html, tags=tags_html)

    @staticmethod
    def get_health_of_features(data):
        """
        :param data: LIST of DICTS aka
                     [{"category": "Feature A", "value": 95, "full": 100},
                      {"category": "Feature B", "value": 90, "full": 100}]
        :return: STRING, HTML for all the Health of Features chart
        """
        def css():

            return """
                    <style>
                        #health {
                            width: 100%;
                            height: 282px;
                        }
                    </style>
                   """

        def js():

            return """
                    <script>
                    $(document).ready(function() {
                        // Themes begin
                        am4core.useTheme(am4themes_dark);
                        am4core.useTheme(am4themes_animated);

                        // Create chart instance
                        var chart = am4core.create("health", am4charts.RadarChart);

                        // Add data
                        chart.data = {data};

                        // Make chart not full circle
                        chart.startAngle = -90;
                        chart.endAngle = 180;
                        chart.innerRadius = am4core.percent(20);

                        // Set number format
                        chart.numberFormatter.numberFormat = "#.#'%'";

                        // Create axes
                        var categoryAxis = chart.yAxes.push(new am4charts.CategoryAxis());
                        categoryAxis.dataFields.category = "category";
                        categoryAxis.renderer.grid.template.location = 0;
                        categoryAxis.renderer.grid.template.strokeOpacity = 0;
                        categoryAxis.renderer.labels.template.horizontalCenter = "right";
                        categoryAxis.renderer.labels.template.fontWeight = 500;
                        categoryAxis.renderer.labels.template.adapter.add("fill", function(fill, target) {
                          return (target.dataItem.index >= 0) ? chart.colors.getIndex(target.dataItem.index) : fill;
                        });
                        categoryAxis.renderer.minGridDistance = 10;

                        var valueAxis = chart.xAxes.push(new am4charts.ValueAxis());
                        valueAxis.renderer.grid.template.strokeOpacity = 0;
                        valueAxis.min = 0;
                        valueAxis.max = 100;
                        valueAxis.strictMinMax = true;

                        // Create series
                        var series1 = chart.series.push(new am4charts.RadarColumnSeries());
                        series1.dataFields.valueX = "full";
                        series1.dataFields.categoryY = "category";

                        series1.columns.template.fill = new am4core.InterfaceColorSet().getFor("alternativeBackground");
                        series1.columns.template.fillOpacity = 0.08;
                        series1.columns.template.cornerRadiusTopLeft = 20;
                        series1.columns.template.strokeWidth = 0;
                        series1.columns.template.radarColumn.cornerRadius = 20;

                        var series2 = chart.series.push(new am4charts.RadarColumnSeries());
                        series2.dataFields.valueX = "value";
                        series2.dataFields.categoryY = "category";
                        series2.clustered = false;
                        series2.columns.template.strokeWidth = 0;
                        series2.columns.template.tooltipText = "{category}: [bold]{value}%[/]";
                        series2.columns.template.radarColumn.cornerRadius = 20;

                        series2.columns.template.adapter.add("fill", function(fill, target) {
                          return chart.colors.getIndex(target.dataItem.index);
                        });

                        // Add cursor
                        chart.cursor = new am4charts.RadarCursor();
                    });
                    </script>
                   """.replace("{data}", str(data))

        return """
                <div class='col s12 m12 l6 xl6'>
                  <div class='card blue-grey darken-3'>
                    <div class='large-data-card card-content white-text'>
                      <span class='card-title'>Health of Features:</span>
                      <div id="health"></div>
                      {css}
                      {js}
                    </div>
                  </div>
                </div>
               """.format(css=css(), js=js())

    @staticmethod
    def get_suggestions(data):

        def css():

            return """
                    <style>
                        #insights {
                            width: 100%;
                            height: 275px;
                            overflow-y: auto;
                        }
                        
                        #insights_list > li {
                            padding: 10px;
                        }
                        
                        #insights_list > li:hover {
                            background: #1e4254;
                            border-radius: 3px;
                        }
                        
                        .traceback-tooltip{
                            background-color: #eae8e8;
                            border-radius: 3px;
                            color: #444;
                            cursor: pointer;
                        }
                    </style>
                   """

        def js():

            return """
                   <script>
                       var insights = {data}
                       var insights_list = $("#insights_list")
                       var insights_html = ""
                       for(index in insights){{
                           insights_html += "<li>"+insights[index]+"</li>"
                       }}
                       insights_list.html(insights_html)
                   </script>
                   """

        return """
                <div class="col s12 m12 l6 xl6">
                  <div class="card blue-grey darken-3">
                    <div class="large-data-card card-content white-text">
                      <span class="card-title">Insights:</span>
                      <div id="insights">
                      <ul id="insights_list">
                      </ul>
                      </div>
                      {css}
                      {js}
                    </div>
                  </div>
                </div>
               """.format(css=css(), js=js().format(data=data))



import traceback

from test_junkie.constants import TestCategory
from test_junkie.debugger import LogJunkie


class XmlReporter:

    @staticmethod
    def create_xml_report(write_file, suites):

        def __update_tag_stats(tag, status):

            tag.set("tests", str(int(suite.get("tests")) + 1))
            if status == TestCategory.SUCCESS:
                tag.set("passed", str(int(tag.get("passed")) + 1))
            else:
                tag.set("failures", str(int(tag.get("failures")) + 1))
            return tag

        if write_file is not None:
            try:
                import os
                from xml.etree.ElementTree import ElementTree, Element, SubElement
                import xml
                if not os.path.exists(write_file):
                    request = Element("root")
                    ElementTree(request).write(write_file)

                xml_file = xml.etree.ElementTree.parse(write_file)
                root = xml_file.getroot()

                for suite_object in suites:

                    test_suite = suite_object.get_class_name()
                    tests = suite_object.get_test_objects()

                    for test_object in tests:

                        test_name = test_object.get_function_name()
                        test_metrics = test_object.metrics.get_metrics()

                        for class_param, class_param_data in test_metrics.items():
                            for param, param_data in class_param_data.items():

                                test_status = param_data["status"]
                                if test_status != TestCategory.SUCCESS:
                                    test_status = "failure"
                                suite_found = False

                                for suite in root.iter("testsuite"):
                                    suite_found = suite.attrib["name"] == test_suite
                                    if suite_found:
                                        __update_tag_stats(suite, test_status)
                                        test = Element("testcase", name=str(test_name), status=str(test_status))
                                        if test_status == "failure":
                                            failure = Element("failure", type="failure")
                                            test.append(failure)
                                        suite.append(test)
                                        ElementTree(root).write(write_file)
                                        break

                                if not suite_found:
                                    suite = SubElement(root, "testsuite", name=test_suite,
                                                       tests="0", passed="0", failures="0")
                                    __update_tag_stats(suite, test_status)
                                    test = SubElement(suite, "testcase", name=str(test_name), status=str(test_status))
                                    if test_status == "failure":
                                        SubElement(test, "failure", type="failure")
                                    ElementTree(root).write(write_file)
            except:
                LogJunkie.error(traceback.format_exc())



from test_junkie.decorators import test, beforeTest, Suite
from tests.junkie_suites.TestListener import TestListener


@Suite(retry=2, listener=TestListener)
class BeforeTestAssertionSuite:

    @beforeTest()
    def before_test(self):
        raise AssertionError("Assertion Error in before test")

    @test(retry=2)
    def ignore_1(self):
        pass

    @test(retry=2)
    def ignore_2(self):
        pass



from test_junkie.constants import TestCategory
from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.SkipSuites import SkipTests


def test_component():

    runner = Runner([SkipTests])
    aggregator = runner.run(tests=[SkipTests.test_1, SkipTests.test_2])
    metrics = aggregator.get_basic_report()["tests"]

    assert metrics[TestCategory.SUCCESS] == 2
    assert metrics[TestCategory.SKIP] == 1

    suites = runner.get_executed_suites()
    metrics = suites[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics, expected_status="success")



from test_junkie.constants import TestCategory
from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.SkipSuite import SkipSuiteAdvanced


def test_component():
    runner = Runner([SkipSuiteAdvanced])
    aggregator = runner.run()
    metrics = aggregator.get_basic_report()["tests"]
    assert metrics[TestCategory.SUCCESS] == 4
    assert metrics[TestCategory.SKIP] == 4
    suites = runner.get_executed_suites()

    metrics = suites[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics, expected_status="success")



import subprocess
import sys
import traceback


class Cmd:

    def __init__(self):

        pass

    @staticmethod
    def run(cmd):

        try:
            try:
                out = subprocess.check_output(cmd)
            except subprocess.CalledProcessError as ex:
                out = ex.output
            if sys.version_info[0] < 3:
                output = str(out).replace("\r", "").replace("\t", "").strip().split("\n")
            else:
                output = str(out).replace("\\r", "").replace("\\t", "").strip().split("\\n")
            for i in list(output):
                if i == "" or len(i) < 2:
                    output.remove(i)
            return output
        except:
            raise Exception("Command: {} produced exception: {}".format(cmd, traceback.format_exc()))



import ast
import imp
import inspect
import os
import sys
import threading
import time
import re
from contextlib import contextmanager

from setuptools.glob import glob

from test_junkie.cli.cli import CliUtils
from test_junkie.constants import CliConstants, Undefined, DocumentationLinks
from test_junkie.debugger import suppressed_stdout
from test_junkie.decorators import synchronized
from test_junkie.errors import BadCliParameters
from test_junkie.runner import Runner
from test_junkie.cli.cli_config import Config


class CliRunner:

    __REGEX_ALIAS_IMPORT = ".*?from test_junkie.decorators import(.*?)Suite as.*?\n"
    __REGEX_NO_ALIAS_IMPORT = ".*?from test_junkie.decorators import(.*?)Suite.*?\n"
    __SCANNER_THREADS = []

    def __init__(self, sources, ignore, suites, **kwargs):

        self.__sources = sources
        self.__code_cov = kwargs.get("code_cov", Undefined)
        self.__cov_rcfile = kwargs.get("cov_rcfile", Undefined)
        self.__guess_root = kwargs.get("guess_root", False)

        self.tjignore = ignore
        self.detected_suites = {}
        self.suites = []
        self.requested_suites = suites
        self.config = Config(config_name=CliConstants.TJ_CONFIG_NAME)

    @property
    def sources(self):
        if self.__sources == Undefined:
            self.__sources = ast.literal_eval(self.config.get_value("sources"))
        if self.__sources == Undefined or not isinstance(self.__sources, list):
            raise BadCliParameters("Sources is a required parameter. You can set it in the config via tj config "
                                   "update -s / --sources to persist or pass it in directly to the command you "
                                   "are running via -s / --sources")
        return self.__sources

    @property
    def code_cov(self):
        if self.__code_cov == Undefined:
            self.__code_cov = ast.literal_eval(self.config.get_value("code_cov", default=False))
        return self.__code_cov

    @property
    def cov_rcfile(self):
        if self.__cov_rcfile == Undefined:
            self.__cov_rcfile = self.config.get_value("cov_rcfile", default=None)
            if self.__cov_rcfile == "None":
                self.__cov_rcfile = None
        return self.__cov_rcfile

    @property
    def guess_root(self):
        if self.__guess_root == Undefined:
            self.__guess_root = self.config.get_value("guess_root", default=False)
        return self.__guess_root

    @staticmethod
    def start_in_a_thread(target, args):

        def get_active_count():
            active = 0
            for thread in CliRunner.__SCANNER_THREADS:
                if thread.isAlive():
                    active += 1
                else:
                    CliRunner.__SCANNER_THREADS.remove(thread)
            return active

        while get_active_count() > 50:
            time.sleep(1)
        new_thread = threading.Thread(target=target, args=args)
        CliRunner.__SCANNER_THREADS.append(new_thread)
        new_thread.start()
        return new_thread

    def __find_and_register_suite(self, _suite_alias, _source, _file_path):

        def guess_project_root(path, _module_name, error):

            print("[{status}] Import error: {error}"
                  .format(status=CliUtils.format_color_string(value="WARNING", color="yellow"),
                          error=error))
            possibility = "{}".format(os.sep).join(path.split(os.sep)[:-1])
            print("[{status}] Trying again with assumption that this is your project root: {assumed_root}"
                  .format(status=CliUtils.format_color_string(value="WARNING", color="yellow"),
                          assumed_root=CliUtils.format_color_string(value=possibility, color="yellow")))
            try:
                sys.path.insert(0, possibility)
                return imp.load_source(_module_name, _file_path)
            except KeyboardInterrupt:
                print("(Ctrl+C) Exiting!")
                exit(12)
            except ImportError as error:
                if len(possibility.split(os.sep)) > 2:
                    return guess_project_root(possibility, _module_name, error)
                raise

        @synchronized()
        def load_module(_decorated_classes):

            module_name = os.path.splitext(os.path.basename(_file_path))[0]
            try:
                with suppressed_stdout(suppress=True):
                    module = imp.load_source(module_name, _file_path)
            except ImportError as error:
                if self.guess_root:
                    module = guess_project_root(_file_path, module_name, error)
                else:
                    print("[{status}] There is an import error: {error}"
                          .format(status=CliUtils.format_color_string(value="ERROR", color="red"), error=error))
                    print("[{status}] 1. Make sure you have installed all of the packages required for your project "
                          "to work".format(status=CliUtils.format_color_string(value="ERROR", color="red")))
                    print("[{status}] 2. Make sure the root of your project is in the PYTHONPATH"
                          .format(status=CliUtils.format_color_string(value="ERROR", color="red")))
                    print("[{status}] 3. TJ can try to guess your root directory if you run with --guess-root "
                          "(Usually not recommended)".format(status=CliUtils.format_color_string(value="ERROR",
                                                                                                 color="red")))
                    raise
            for name, data in inspect.getmembers(module):
                if name in _decorated_classes and inspect.isclass(data):
                    if not self.requested_suites or \
                            (self.requested_suites and name in self.requested_suites):
                        self.suites.append(data)

        matches = re.findall("@{alias}((.|\n)*?):\n".format(alias=_suite_alias), _source)
        decorated_classes = []
        for match in matches:
            if isinstance(match, tuple):
                for item in match:
                    if "\nclass " in item:
                        decorated_classes.append(item.split("\nclass ")[-1].strip())
            else:
                if "\nclass " in match:
                    decorated_classes.append(match.split("\nclass ")[-1].strip())
        load_module(decorated_classes)

    def __skip(self, source, directory):

        if source not in directory:
            return True

        for ignored_item in self.tjignore:
            if ignored_item in directory:
                return True
        return False

    def scan(self):

        @contextmanager
        def open_file(_file):
            if sys.version_info[0] < 3:
                with open(_file) as _doc:
                    _source = _doc.read()
                    yield (_source, _doc)
            else:
                with open(_file, encoding="utf-8") as _doc:
                    _source = _doc.read()
                    yield (_source, _doc)

        def parse_file(_file):

            with open_file(_file) as __source:

                suite_imported_as_alias = re.findall(CliRunner.__REGEX_ALIAS_IMPORT, __source[0])
                if suite_imported_as_alias:
                    suite_alias = suite_imported_as_alias[-1].split("Suite")[-1].split("as")[-1].split(",")[0].strip()
                    CliRunner.start_in_a_thread(target=self.__find_and_register_suite,
                                                args=(suite_alias, __source[0], __source[1].name))
                    # self.__find_and_register_suite(suite_alias, __source[0], __source[1].name)
                    return True

                suite_imported = re.findall(CliRunner.__REGEX_NO_ALIAS_IMPORT, __source[0])
                if suite_imported:
                    # self.__find_and_register_suite("Suite", __source[0], __source[1].name)
                    CliRunner.start_in_a_thread(target=self.__find_and_register_suite,
                                                args=("Suite", __source[0], __source[1].name))
                    return True

        try:
            print("\n[{status}] Scanning: {location} ..."
                  .format(location=CliUtils.format_color_string(value=",".join(self.sources), color="green"),
                          status=CliUtils.format_color_string(value="INFO", color="blue")))
            start = time.time()
            for source in self.sources:
                if source.endswith(".py"):
                    parse_file(source)
                else:
                    for dirName, subdirList, fileList in os.walk(source, topdown=True):

                        if self.__skip(source, dirName):
                            continue

                        for file_path in glob(os.path.join(os.path.dirname(dirName+"\\"), "*.py")):
                            if parse_file(file_path)is True:
                                continue
            for thread in CliRunner.__SCANNER_THREADS:
                thread.join()
            print("[{status}] Scan finished in: {time} seconds. Found: {suites} suite(s)."
                  .format(status=CliUtils.format_color_string(value="INFO", color="blue"),
                          time="{0:.2f}".format(time.time() - start),
                          suites=CliUtils.format_bold_string(len(self.suites))))
        except KeyboardInterrupt:
            print("(Ctrl+C) Exiting!")
            exit(12)
        except BadCliParameters as err:
            print("[{status}] {error}.".format(status=CliUtils.format_color_string(value="ERROR", color="red"),
                                               error=err))
            exit(120)
        except:
            print("[{status}] Unexpected error during scan for test suites.".format(
                status=CliUtils.format_color_string(value="ERROR", color="red")))
            CliUtils.print_color_traceback()
            exit(120)

    def run_suites(self, args):

        def tags():
            config = {"run_on_match_all": args.run_on_match_all,
                      "run_on_match_any": args.run_on_match_any,
                      "skip_on_match_all": args.skip_on_match_all,
                      "skip_on_match_any": args.skip_on_match_any}
            for prop, value in config.items():
                if value is not None:
                    return config
            return None

        if self.suites:
            print("[{status}] Running tests ...\n"
                  .format(status=CliUtils.format_color_string(value="INFO", color="blue")))
            if self.code_cov:
                import coverage

                if self.cov_rcfile is not None:
                    print(">>>>", 1)
                    cov = coverage.Coverage(omit="*{sep}test_junkie{sep}*".format(sep=os.sep),
                                            config_file=self.cov_rcfile)
                else:
                    print(">>>>", 2)
                    cov = coverage.Coverage(omit="*{sep}test_junkie{sep}*".format(sep=os.sep))
                cov.start()
            try:
                runner = Runner(suites=self.suites,
                                html_report=args.html_report,
                                xml_report=args.xml_report,
                                config=Config.get_config_path(CliConstants.TJ_CONFIG_NAME))
                runner.run(test_multithreading_limit=args.test_multithreading_limit,
                           suite_multithreading_limit=args.suite_multithreading_limit,
                           tests=args.tests,
                           owners=args.owners,
                           components=args.components,
                           features=args.features,
                           tag_config=tags(),
                           quiet=args.quiet)
            except KeyboardInterrupt:
                print("(Ctrl+C) Exiting!")
                exit(12)
            except:
                print("[{status}] Unexpected error during test execution.".format(
                      status=CliUtils.format_color_string(value="ERROR", color="red")))
                CliUtils.print_color_traceback()
                exit(120)
            finally:
                if self.code_cov:
                    cov.stop()
                    cov.save()
            if self.code_cov:
                try:
                    cov.report(show_missing=True, skip_covered=True)
                    print("[{status}] TJ uses Coverage.py. Control it with --cov-rcfile, "
                          "see {link}".format(status=CliUtils.format_color_string(value="TIP", color="blue"),
                                              link=DocumentationLinks.COVERAGE_CONFIG_FILE))
                except coverage.misc.CoverageException:
                    pass
            return



import time
from random import randint


from test_junkie.decorators import Suite, test, beforeClass, beforeTest, afterTest, afterClass
from test_junkie.meta import meta, Meta
from tests.junkie_suites.TestListener import TestListener


@Suite(retry=2,
       listener=TestListener,
       parameters=[1, 2],
       parallelized=False,
       feature="API")
class AuthApiSuite:

    @test(component="Auth", tags=["api", "auth", "basic_auth"])
    def authenticate_via_basic_auth_api(self):
        pass

    @test(component="Auth", tags=["api", "auth", "two_factor"], owner="Victor")
    def authenticate_via_two_factor_api(self):
        pass

    @test(component="Auth", tags=["api", "auth", "sso"], owner="Victor")
    def authenticate_via_sso(self):
        pass

    @test(component="Auth", tags=["api", "auth", "sso"], owner="Victor")
    def sso_hit_negative_auth_limit(self):
        # time.sleep(randint(0, 4))
        pass

    @test(component="Auth", tags=["api", "auth", "basic_auth"], owner="Victor")
    def auth_hit_negative_auth_limit(self):
        # time.sleep(randint(0, 4))
        pass

    @test(component="Auth", tags=["api", "auth", "two_factor"], owner="Victor")
    def two_factor_hit_negative_auth_limit(self):
        # time.sleep(randint(0, 4))
        pass



import inspect
import threading
import time

from test_junkie.debugger import LogJunkie
from test_junkie.decorators import synchronized


class ParallelProcessor:

    __PARALLELS = {}
    __REVERSE_PARALLEL_RESTRICTIONS = {}

    def __init__(self, settings):

        # self.__test_limit = int(kwargs.get("test_multithreading_limit", 1))
        self.__test_limit = settings.test_thread_limit
        if self.__test_limit == 0 or self.__test_limit is None:
            LogJunkie.warn("Thread limit for tests cannot be 0 or None, "
                           "falling back to limit of 1 thread per test case.")
            self.__test_limit = 1

        # self.__suite_limit = int(kwargs.get("suite_multithreading_limit", 1))
        self.__suite_limit = settings.suite_thread_limit
        if self.__suite_limit == 0 or self.__suite_limit is None:
            LogJunkie.warn("Thread limit for suites cannot be 0 or None, "
                           "falling back to limit of 1 thread per test suite.")
            self.__suite_limit = 1

        LogJunkie.debug("=======================Parallel Processor Settings=============================")
        LogJunkie.debug(">> Suite level multi-threading enabled: {}".format(self.suite_multithreading()))
        LogJunkie.debug(">> Suite level multi-threading limit: {}".format(self.__suite_limit))
        LogJunkie.debug(">> Test level multi-threading enabled: {}".format(self.test_multithreading()))
        LogJunkie.debug(">> Test level multi-threading limit: {}".format(self.__test_limit))
        LogJunkie.debug("===============================================================================")

    def suite_multithreading(self):

        return self.__suite_limit > 1

    def test_multithreading(self):

        return self.__test_limit > 1

    def test_limit_reached(self, parallels):

        active = 0
        for parallel in parallels:
            if parallel.isAlive():
                active += 1
        if active >= self.__test_limit:
            LogJunkie.debug("Test limit: {}/{}".format(active, self.__test_limit))
            return True
        return False

    @staticmethod
    def run_suite_in_a_thread(func, suite):
        thread = threading.Thread(target=func, args=(suite,))
        if suite not in ParallelProcessor.__PARALLELS:
            ParallelProcessor.__PARALLELS.update({suite.get_class_object(): {"thread": thread, "tests": []}})
        thread.start()
        return thread

    @staticmethod
    def wait_for_parallels_to_finish(parallels):
        for parallel in parallels:
            parallel.join()

    @staticmethod
    def run_test_in_a_thread(func, suite, test, parameter, class_parameter, before_class_error, cancel):
        thread = threading.Thread(target=func, args=(suite, test, parameter, class_parameter,
                                                     before_class_error, cancel))
        if suite not in ParallelProcessor.__PARALLELS:
            ParallelProcessor.__PARALLELS.update({suite.get_class_object(): {"thread": thread, "tests": []}})
        ParallelProcessor.__PARALLELS[suite.get_class_object()]["tests"].append({"test": test, "thread": thread})
        thread.start()
        return thread

    @staticmethod
    def get_active_parallels_count():
        active_parallels = 0
        for suite, data in ParallelProcessor.__PARALLELS.items():
            if data["thread"].isAlive():
                active_parallels += 1
        return active_parallels

    @staticmethod
    def get_active_test_parallels_count(suite_object):
        active_parallels = 0
        data = ParallelProcessor.__PARALLELS.get(suite_object.get_class_object(), None)
        if data is not None:
            for test in data["tests"]:
                if test["thread"].isAlive():
                    active_parallels += 1
        return active_parallels

    @staticmethod
    def get_active_parallels():
        return ParallelProcessor.__PARALLELS

    def suite_qualifies(self, suite):

        def _build_reverse_restriction():
            """
            Bidirectional parallel restriction will be automatically added.
            If suite `A` is restricted to run when suite `B` is running - suite `B` will be automatically restricted
            to run when suite `A` is running
            :return: None
            """
            if restriction not in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS:
                ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS.update({restriction: [suite.get_class_object()]})
            elif suite.get_class_object() not in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[restriction]:
                ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[restriction].append(suite.get_class_object())
            else:
                return  # if nothing to add, return - to avoid logging
            LogJunkie.debug("Added reverse restriction! {} will not be processed while {} is running"
                            .format(restriction, suite.get_class_object()))

        def _passes_restriction():
            """
            If current suite does not have any active restrictions, we can run it
            :return: BOOLEAN
            """
            if ParallelProcessor.__PARALLELS.get(restriction, None) is not None:
                if ParallelProcessor.__PARALLELS[restriction]["thread"].isAlive():
                    LogJunkie.debug("Suite: {} can't run while: {} is running."
                                    .format(suite.get_class_object(), restriction))
                    return False
            return True

        def _passes_reverse_restriction():
            """
            If current suite is part of parallel restriction in another suite which is currently active, can't run it.
            :return: BOOLEAN
            """
            if suite.get_class_object() in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS:
                reverse_suites = ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[suite.get_class_object()]
                for reverse_suite in reverse_suites:
                    if ParallelProcessor.__PARALLELS.get(reverse_suite, None) is not None:
                        if ParallelProcessor.__PARALLELS[reverse_suite]["thread"].isAlive():
                            LogJunkie.debug("Suite: {} can't run while: {} is running due to reverse restriction."
                                            .format(suite.get_class_object(), reverse_suite))
                            return False
            return True

        if suite.get_parallel_restrictions():
            for restriction in suite.get_parallel_restrictions():
                if not inspect.isclass(restriction):
                    raise Exception("Parallel suite restrictions must be class objects. "
                                    "Instead suite: {} was restricted by a function: {}"
                                    .format(suite.get_class_object(), restriction))
                _build_reverse_restriction()
                if not _passes_restriction():
                    return False
        if not _passes_reverse_restriction():
            return False

        if self.__suite_limit is not None:
            while ParallelProcessor.get_active_parallels_count() >= self.__suite_limit:
                LogJunkie.debug("Suite level Thread limit reached! Active suites: {}/{}"
                                .format(ParallelProcessor.get_active_parallels_count(), self.__suite_limit))
                time.sleep(5)
        return True

    @synchronized(threading.Lock())
    def test_qualifies(self, suite, test):

        def _build_reverse_restriction():
            """
            Bidirectional parallel restriction will be automatically added.
            If suite `A` is restricted to run when suite `B` is running - suite `B` will be automatically restricted
            to run when suite `A` is running
            :return: None
            """
            if restriction not in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS:
                ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS.update({restriction: [test.get_function_object()]})
            elif test.get_function_object() not in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[restriction]:
                ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[restriction].append(test.get_function_object())
            else:
                return  # if nothing to add, return - to avoid logging
            LogJunkie.debug("Added reverse test restriction! {} will not be processed while test: {} is running"
                            .format(restriction, test.get_function_object()))

        def _passes_restriction():
            """
            If current suite does not have any active restrictions, we can run it
            :return: BOOLEAN
            """
            for class_object, suite_mapping in list(ParallelProcessor.__PARALLELS.items()):
                for test_mapping in suite_mapping["tests"]:
                    if test_mapping["test"].get_function_object() in test.get_parallel_restrictions():
                        if test_mapping["thread"].isAlive():
                            return False
            return True

        def _passes_reverse_restriction():
            """
            If current suite is part of parallel restriction in another suite which is currently active, can't run it.
            :return: BOOLEAN
            """
            if test.get_function_object() in ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS:
                reverse_tests = ParallelProcessor.__REVERSE_PARALLEL_RESTRICTIONS[test.get_function_object()]
                for class_object, suite_mapping in list(ParallelProcessor.__PARALLELS.items()):
                    for test_mapping in suite_mapping["tests"]:
                        if test_mapping["test"].get_function_object() in reverse_tests:
                            if test_mapping["thread"].isAlive():
                                return False
            return True

        if test.get_parallel_restrictions():
            for restriction in test.get_parallel_restrictions():
                if not inspect.isfunction(restriction) and not inspect.ismethod(restriction):
                    raise Exception("Parallel test restrictions must be function objects. "
                                    "Instead test: {} was restricted by: {}"
                                    .format(test.get_function_name(), restriction))
                _build_reverse_restriction()
                if not _passes_restriction():
                    return False
        if not _passes_reverse_restriction():
            return False

        if self.__test_limit is not None:
            while ParallelProcessor.get_active_test_parallels_count(suite) >= self.__test_limit:
                LogJunkie.debug("Test level Thread limit reached! Active tests: {}/{}"
                                .format(ParallelProcessor.get_active_test_parallels_count(suite), self.__test_limit))
                time.sleep(5)
        return True



import time

from test_junkie.decorators import test, Suite, afterClass, afterTest, beforeTest, beforeClass


@Suite(feature="Login", owner="Mike")
class Login:

    @test(component="Authentication", owner="John", tags=["awesome", "tags"])
    def positive_login(self):
        time.sleep(1)

    @test(component="Authentication", owner="John")
    def negative_login(self):
        time.sleep(1)

    @test(component="Login Inputs", tags=["awesome", "tags"])
    def login_no_password(self):
        time.sleep(1)

    @test(component="Login Inputs")
    def login_no_username(self):
        time.sleep(1)


@Suite(feature="Login", owner="John")
class LoginSessions:

    @test(component="Session Timeout")
    def session_timeout(self):
        time.sleep(1)

    @test(component="Session Timeout", tags=["awesome", "tags"])
    def positive_login_on_expired_session(self):
        time.sleep(1)


@Suite(feature="Dashboard", owner="Jane", retry=3)
class Dashboard:

    @beforeClass()
    def before_class(self):
        time.sleep(1)

    @beforeTest()
    def before_test(self):
        pass

    @afterTest()
    def after_test(self):
        pass

    @afterClass()
    def after_class(self):
        time.sleep(1)

    @test(component="Charts")
    def add_chart(self):
        time.sleep(1)

    @test(component="Charts", tags=["awesome", "tags"])
    def remove_chart(self):
        time.sleep(1)

    @test(component="Charts", tags=["awesome", "tags"])
    def remove_chart(self):
        time.sleep(1)
        raise Exception("Test")

    @test(component="Charts", tags=["awesome", "tags"])
    def remove_chart(self):
        time.sleep(1)
        raise AssertionError("Test")




class Rules:

    def __init__(self, **kwargs):

        self.kwargs = kwargs

    def before_class(self):
        pass

    def before_test(self, **kwargs):
        pass

    def after_test(self, **kwargs):
        pass

    def after_class(self):
        pass



# -*- encoding: utf-8 -*-
import datetime
import time
from random import randint

from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener


@Suite(retry=2,
       listener=TestListener,
       meta={"name": "Suite B", "known_bugs": []},
       parameters=[1, 2], priority=1, feature="Store",
       owner="George")
class ShoppingCartSuite:

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def add_to_cart(self):
        print(datetime.datetime.now())
        print("асдфсадфа")
        # time.sleep(randint(0, 7))
        print("Finished SUITE B / TEST A")

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def remove_from_cart(self):
        # time.sleep(randint(0, 4))
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def increase_quantity(self):
        # time.sleep(randint(0, 4))
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def decrease_quantity(self):
        # time.sleep(randint(0, 4))
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def save_for_later(self):
        # time.sleep(randint(0, 4))
        pass



from test_junkie.decorators import GroupRules, beforeGroup, afterGroup
from test_junkie.objects import TestObject, SuiteObject
from test_junkie.rules import Rules


class TestRules(Rules):

    def __init__(self, **kwargs):

        Rules.__init__(self, **kwargs)

    def before_class(self):
        assert isinstance(self.kwargs.get("suite"), SuiteObject), "Suite Object must be passed to the before_test()"

    def before_test(self, **kwargs):
        assert isinstance(kwargs.get("test"), TestObject), "Test Object must be passed to the before_test()"
        assert isinstance(self.kwargs.get("suite"), SuiteObject), "Suite Object must be passed to the before_test()"

    def after_test(self, **kwargs):
        assert isinstance(kwargs.get("test"), TestObject), "Test Object must be passed to the before_test()"
        assert isinstance(self.kwargs.get("suite"), SuiteObject), "Suite Object must be passed to the before_test()"

    def after_class(self):
        assert isinstance(self.kwargs.get("suite"), SuiteObject), "Suite Object must be passed to the before_test()"

    @GroupRules()
    def group_rules(self):

        from tests.junkie_suites.AdvancedSuite import AdvancedSuite

        @beforeGroup([AdvancedSuite])
        def before_group():
            pass

        @afterGroup([AdvancedSuite])
        def after_group():
            raise Exception("Test exception")

        from tests.junkie_suites.IgnoreSuite import IgnoreSuiteBeforeGroupRule
        from tests.junkie_suites.IgnoreSuite import IgnoreSuiteBeforeGroupRule2

        @beforeGroup([IgnoreSuiteBeforeGroupRule, IgnoreSuiteBeforeGroupRule2])
        def before_group():
            assert True is False



import pprint
import sys
sys.path.insert(1, __file__.split("tests")[0])
from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.ParallelSuiteA import ParallelSuiteA
from tests.junkie_suites.ParallelSuiteB import ParallelSuiteB
from tests.junkie_suites.ParallelSuiteC import ParallelSuiteC

runner = Runner([ParallelSuiteA, ParallelSuiteB, ParallelSuiteC])
runner.run(test_multithreading_limit=2, suite_multithreading_limit=2)
results = runner.get_executed_suites()

for suite_object in results:
    print(suite_object.get_class_name())
    pprint.pprint(suite_object.metrics.get_metrics())


def test_class_metrics():

    assert results[0].get_class_name() == "ParallelSuiteA"
    metrics = results[0].metrics.get_metrics()
    assert int(metrics["runtime"]) < int(results[1].metrics.get_metrics()["runtime"])
    assert int(metrics["runtime"]) < int(results[2].metrics.get_metrics()["runtime"])
    QualityManager.check_class_metrics(metrics,
                                       expected_status="success",
                                       expected_retry_count=1)


def test_test_metrics():

    assert results[0].get_test_objects()
    for test in results[0].get_test_objects():
        for class_param, class_data in test.metrics.get_metrics().items():
            for param, metrics in class_data.items():
                QualityManager.check_test_metrics(metrics,
                                                  expected_status="success",
                                                  expected_param=param,
                                                  expected_class_param=metrics["class_param"])


def test_class_metrics2():

    assert results[1].get_class_name() == "ParallelSuiteB"
    metrics = results[1].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="success",
                                       expected_retry_count=1)


def test_test_metrics2():

    assert results[1].get_test_objects()
    for test in results[1].get_test_objects():
        for class_param, class_data in test.metrics.get_metrics().items():
            for param, metrics in class_data.items():
                QualityManager.check_test_metrics(metrics,
                                                  expected_status="success",
                                                  expected_param=param,
                                                  expected_class_param=metrics["class_param"])


def test_class_metrics3():

    assert results[2].get_class_name() == "ParallelSuiteC"
    metrics = results[2].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="success",
                                       expected_retry_count=1)


def test_test_metrics3():

    assert results[2].get_test_objects()
    for test in results[2].get_test_objects():
        for class_param, class_data in test.metrics.get_metrics().items():
            for param, metrics in class_data.items():
                QualityManager.check_test_metrics(metrics,
                                                  expected_status="success",
                                                  expected_param=param,
                                                  expected_class_param=metrics["class_param"])



from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener


@Suite(retry=2,
       listener=TestListener,
       parameters=[1, 2],
       parallelized=False,
       feature="API")
class AuthApiSuite:

    @test(component="Auth", tags=["api", "auth", "basic_auth"], owner="Victor")
    def authenticate_via_basic_auth_api(self):
        pass

    @test(component="Auth", tags=["api", "auth", "two_factor"], owner="Victor")
    def authenticate_via_two_factor_api(self):
        pass

    @test(component="Auth", tags=["api", "auth", "sso"], owner="Mike")
    def authenticate_via_sso(self):
        pass

    @test(component="Auth", tags=["api", "auth", "sso"], owner="Mike")
    def sso_hit_negative_auth_limit(self):
        pass

    @test(component="Auth", tags=["api", "auth", "basic_auth"], owner="Victor")
    def auth_hit_negative_auth_limit(self):
        pass

    @test(component="Auth", tags=["api", "auth", "two_factor"], owner="Victor")
    def two_factor_hit_negative_auth_limit(self):
        pass


@Suite(retry=2,
       listener=TestListener,
       meta={"name": "Suite B", "known_bugs": []},
       parameters=[1, 2], priority=1, feature="Store",
       owner="George")
class ShoppingCartSuite:

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def add_to_cart(self):
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def remove_from_cart(self):
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def increase_quantity(self):
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def decrease_quantity(self):
        pass

    @test(priority=1,
          component="Shopping Cart",
          tags=["ui", "cart", "positive_flow"])
    def save_for_later(self):
        pass


@Suite(retry=2,
       listener=TestListener,
       priority=2, feature="Store", owner="Mike")
class NewProductsSuite:

    @test(component="Admin", tags=["store_management"])
    def add_new_product(self):
        pass

    @test(component="Admin", tags=["store_management"])
    def remove_product(self):
        pass

    @test(component="Admin", tags=["store_management"])
    def publish_product(self):
        pass

    @test(component="Admin", tags=["store_management"])
    def edit_product(self):
        pass

    @test(component="Admin", tags=["store_management"])
    def archive_product(self):
        pass



import pprint

from test_junkie.errors import BadParameters, BadSignature
from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.IgnoreSuite import IgnoreSuiteBoundMethod, IgnoreSuiteFunction, IgnoreSuiteClassic, \
    IgnoreSuiteClassic2, IgnoreSuiteClassic3
from tests.junkie_suites.error_handling.ErrorSuite4 import ErrorSuite4
from tests.junkie_suites.error_handling.ErrorSuite5 import ErrorSuite5

runner1 = Runner([IgnoreSuiteBoundMethod])
runner1.run()
results1 = runner1.get_executed_suites()

pprint.pprint(results1[0].metrics.get_metrics())
for test in results1[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics():

    metrics = results1[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="ignore",
                                       expected_retry_count=0)


def test_test_metrics():

    assert results1[0].get_test_objects()
    for test in results1[0].get_test_objects():

        properties = test.metrics.get_metrics()
        assert len(properties) == 0


runner2 = Runner([IgnoreSuiteFunction])
runner2.run()
results2 = runner2.get_executed_suites()

pprint.pprint(results2[0].metrics.get_metrics())
for test in results2[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics2():

    metrics = results2[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="ignore",
                                       expected_retry_count=0)


def test_test_metrics2():

    assert results2[0].get_test_objects()
    for test in results2[0].get_test_objects():

        properties = test.metrics.get_metrics()
        assert len(properties) == 0


runner3 = Runner([IgnoreSuiteClassic])
runner3.run()
results3 = runner3.get_executed_suites()

pprint.pprint(results3[0].metrics.get_metrics())
for test in results3[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics3():

    metrics = results3[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="ignore",
                                       expected_retry_count=0)


def test_test_metrics3():

    assert results3[0].get_test_objects()
    for test in results3[0].get_test_objects():

        properties = test.metrics.get_metrics()
        assert len(properties) == 0


runner4 = Runner([IgnoreSuiteClassic2])
runner4.run()
results4 = runner4.get_executed_suites()

pprint.pprint(results4[0].metrics.get_metrics())
for test in results4[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics4():

    metrics = results4[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="ignore",
                                       expected_retry_count=0)


def test_test_metrics4():

    assert results4[0].get_test_objects()
    for test in results4[0].get_test_objects():

        properties = test.metrics.get_metrics()
        assert len(properties) == 0


runner = Runner([IgnoreSuiteClassic3])
runner.run()
results = runner.get_executed_suites()

pprint.pprint(results[0].metrics.get_metrics())
for test in results[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics5():

    metrics = results[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="ignore",
                                       expected_retry_count=0)


def test_test_metrics5():

    assert results[0].get_test_objects()
    for test in results[0].get_test_objects():

        properties = test.metrics.get_metrics()
        assert len(properties) == 0


def test_wrong_params():

    try:
        from tests.junkie_suites.error_handling import ErrorSuite1
        raise AssertionError("This test must raise exception, as wrang datatype is used for parameters")
    except Exception as error:
        assert isinstance(error, BadParameters), "Exception must be raised for bad parameters"


def test_wrong_params2():

    try:
        from tests.junkie_suites.error_handling import ErrorSuite2
        raise AssertionError("This test must raise exception, as wrang datatype is used for parameters")
    except Exception as error:
        assert isinstance(error, BadSignature), "Exception must be raised for bad signature"


def test_wrong_params3():

    try:
        from tests.junkie_suites.error_handling.ErrorSuite3 import ErrorSuite3
        raise AssertionError("This test must raise exception, as wrang datatype is used for parameters")
    except Exception as error:
        assert isinstance(error, BadParameters), "Exception must be raised for bad parameters"


runner5 = Runner([ErrorSuite4])
runner5.run()
results5 = runner5.get_executed_suites()

pprint.pprint(results5[0].metrics.get_metrics())
for test in results5[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_before_test_error1():

    metrics5 = results5[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics5,
                                       expected_status="fail",
                                       expected_retry_count=1,
                                       expected_beforetest_performance_count=1,
                                       expected_beforetest_exception_object=Exception,
                                       expected_beforetest_exception_count=1)


def test_before_test_error2():

    assert results5[0].get_test_objects()
    for test in results5[0].get_test_objects():

        if test.get_function_name() == "failure":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_status="error",
                                              expected_exception=Exception)


runner6 = Runner([ErrorSuite5])
runner6.run()
results6 = runner6.get_executed_suites()

pprint.pprint(results6[0].metrics.get_metrics())
for test in results6[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_after_test_error3():

    metrics6 = results6[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics6,
                                       expected_status="fail",
                                       expected_retry_count=1,
                                       expected_aftertest_performance_count=1,
                                       expected_aftertest_exception_object=Exception,
                                       expected_aftertest_exception_count=1)


def test_after_test_error4():

    assert results6[0].get_test_objects()
    for test in results6[0].get_test_objects():

        if test.get_function_name() == "failure":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_status="error",
                                              expected_exception=Exception)



import collections

from test_junkie.builder import Builder
from test_junkie.constants import Undefined


class CliAudit:

    __SECTIONS = ["owners", "features", "suites", "components", "tags"]

    def __init__(self, suites, args):

        self.aggregated_data = {
                "absolute_test_count": 0,  # parameterized tests will be treated as 1 test
                "absolute_suite_count": 0,

                "context_by_features": {},
                "context_by_owners": {},
                "context_by_suites": {},
                "context_by_tags": {},
                "context_by_components": {},

                "parameterized_test_count": 0,
                "parameterized_suite_count": 0,
                }
        self.suites = suites
        self.exe_roster = Builder.get_execution_roster()
        self.args = args

    def aggregate(self):

        def is_relevant(_suite=None, _test=None):
            if not _suite and not _test:
                raise Exception("Must pass in either a SuiteObject or a TestObject!")
            elif _suite and not _test:
                from test_junkie.rules import Rules
                if self.args.no_rules and _suite.get_rules().__class__ != Rules:
                    return False

                from test_junkie.listener import Listener
                if self.args.no_listeners and _suite.get_listener().__class__ != Listener:
                    return False

                if self.args.no_suite_retries and _suite.get_retry_limit() > 1:
                    return False

                if self.args.no_suite_meta and _suite.get_meta():
                    return False

                if self.args.no_owners and _suite.get_owner():
                    return False

                if self.args.no_features and _suite.get_feature():
                    return False

                if self.args.features != Undefined:
                    if _suite.get_feature() in self.args.features:
                        return True
                    return False
            else:
                if self.args.no_test_retries and _test.get_retry_limit() > 1:
                    return False

                if self.args.no_test_meta and _suite.get_meta():
                    return False

                if self.args.no_owners and _test.get_owner():
                    return False

                if self.args.no_components and _test.get_component():
                    return False

                if self.args.no_tags and _test.get_tags():
                    return False

                if self.args.owners != Undefined:
                    if _test.get_owner() in self.args.owners:
                        return True
                    return False

                if self.args.components != Undefined:
                    if _test.get_component() in self.args.components:
                        return True
                    return False

                if self.args.tags != Undefined:
                    for tag in self.args.tags:
                        if tag in _test.get_tags():
                            return True
                    return False
            return True

        for suite, suite_object in self.exe_roster.items():
            if not is_relevant(_suite=suite_object):
                continue

            tests = suite_object.get_test_objects()
            self.aggregated_data["absolute_suite_count"] += 1
            if suite_object.get_parameters() != [None]:
                self.aggregated_data["parameterized_suite_count"] += 1

            feature = suite_object.get_feature()
            if feature not in self.aggregated_data["context_by_features"]:
                self.aggregated_data["context_by_features"].update({
                    feature: {"tags": {}, "owners": {}, "components": {}, "suites": {}, "total_tests": 0}})
            feature_context = self.aggregated_data["context_by_features"][feature]
            feature_context["total_tests"] += len(tests)
            feature_context["suites"].update({"{}.{}".format(suite_object.get_class_module(),
                                                             suite_object.get_class_name()): len(tests)})

            suite_context = {"total_tests": len(tests),
                             "tags": {},
                             "owners": {},
                             "components": {},
                             "feature": feature}

            self.aggregated_data["absolute_test_count"] += len(tests)
            for test in tests:

                if test.get_owner() is None:
                    test.get_kwargs().update({"owner": suite_object.get_owner()})

                if not is_relevant(_suite=suite, _test=test):
                    continue
                # self.aggregated_data["absolute_test_count"] += 1
                if test.accepts_suite_parameters() or test.accepts_test_parameters():
                    self.aggregated_data["parameterized_test_count"] += 1

                for context in [feature_context, suite_context]:
                    CliAudit.process_tags(context, test)
                    CliAudit.process_property(context, "components", test.get_component())
                    CliAudit.process_property(context, "owners", test.get_owner())

                self.update_context(suite, test, feature)
            self.aggregated_data["context_by_suites"].update({suite_object.get_class_name(): suite_context})

    @staticmethod
    def process_property(data_context, prop, key):
        if key not in data_context[prop]:
            data_context[prop].update({key: 1})
        else:
            data_context[prop][key] += 1

    @staticmethod
    def process_tags(data_context, test):
        for tag in test.get_tags():
            if tag not in data_context["tags"]:
                data_context["tags"].update({tag: 1})
            else:
                data_context["tags"][tag] += 1

    def update_context(self, suite, test, feature):

        def process_data_context(_data_context, _context):

            if _context != "tags":
                CliAudit.process_tags(_data_context, test)
            if _context != "components":
                CliAudit.process_property(_data_context, "components", test.get_component())
            if _context != "owners":
                CliAudit.process_property(_data_context, "owners", test.get_owner())

            CliAudit.process_property(_data_context, "features", feature)
            CliAudit.process_property(_data_context, "suites", "{}.{}".format(suite.__module__, suite.__name__))

        def get_template(_context):
            template = {"total_tests": 0}
            for attribute in CliAudit.__SECTIONS:
                if attribute != _context:
                    template.update({attribute: {}})
            return template

        def update_context_template(_value, _context):

            if _value not in self.aggregated_data["context_by_{context}".format(context=context)]:
                self.aggregated_data["context_by_{context}".format(context=context)].update(
                    {_value: get_template(_context)})
            data_context = self.aggregated_data["context_by_{context}".format(context=context)][_value]
            data_context["total_tests"] += 1
            process_data_context(data_context, _context)

        for context in ["tags", "components", "owners"]:
            if context == "tags":
                for tag in test.get_tags():
                    update_context_template(tag, context)
            else:
                value = test.get_component() if context == "components" else test.get_owner()
                update_context_template(value, context)

    def print_results(self):

        from test_junkie.cli.cli import CliUtils
        match_found = False
        output = []
        for data_context in CliAudit.__SECTIONS:
            if data_context == self.args.command:
                data = self.aggregated_data["context_by_{context}".format(context=data_context)]
                if data:
                    section = []
                    _sorted_data = sorted(data.items(), key=lambda x: x[1]["total_tests"])
                    _sorted_data.reverse()
                    _sorted_data = collections.OrderedDict(_sorted_data)
                    for primary_key, context in _sorted_data.items():
                        details = []
                        if data_context == "suites":
                            details.append("\nSuite: {value} Feature: {feature}"
                                           .format(value=CliUtils.format_bold_string(primary_key),
                                                   feature=CliUtils.format_bold_string(context["feature"])))
                        else:
                            parent = "".join(list(data_context)[:-1]).capitalize()  # exp: features > Feature
                            if primary_key is None:
                                primary_key = CliUtils.format_color_string(primary_key, "red")
                            else:
                                primary_key = CliUtils.format_bold_string(primary_key)
                            details.append("\n{parent}: {value}".format(parent=parent, value=primary_key))

                        from test_junkie.metrics import Aggregator
                        details.append("\t- Tests:\t{total} of {absolute} total tests ({percentage}%)"
                                       .format(total=context["total_tests"],
                                               absolute=self.aggregated_data["absolute_test_count"],
                                               percentage=Aggregator.percentage(
                                                   self.aggregated_data["absolute_test_count"],
                                                   context["total_tests"])))

                        for i in CliAudit.__SECTIONS:
                            if i in context:
                                msg = "\t"
                                counter = 0
                                _sorted_context = sorted(context[i].items(), key=lambda x: x[1])
                                _sorted_context.reverse()
                                _sorted_context = collections.OrderedDict(_sorted_context)
                                for key, count in _sorted_context.items():
                                    if counter > 0:
                                        msg += "\n\t\t\t"
                                    if key is None:
                                        key = CliUtils.format_color_string(key, "red")
                                    msg += "{} ({})".format(key, count)
                                    counter += 1
                                if len(msg) > 0:
                                    details.append("\t- {i}: {msg}".format(i=i.capitalize(), msg=msg))
                        if len(details) >= 5:
                            section += details
                    if len(section) > 1:
                        output.append(section)
                    break

        for section in output:
            if len(section) >= 5:
                match_found = True
                for msg in section:
                    print(msg)

        if not match_found:
            print("[{status}] Nothing matches your search criteria!"
                  .format(status=CliUtils.format_color_string("INFO", "blue")))



from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener


@Suite(listener=TestListener, parameters={1, 2}, retry=2)
class ErrorSuite1:

    @test()
    def test_1(self, class_parameter):
        pass



from test_junkie.listener import Listener


class TestListener(Listener):

    def on_cancel(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_success(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_failure(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_skip(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_error(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_ignore(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_in_progress(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_complete(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "test_meta" in kwargs["properties"], "missing test meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jto" in kwargs["properties"]["jm"], "missing test object in junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_before_class_error(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_before_class_failure(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_after_class_error(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_after_class_failure(self, **kwargs):
        assert "exception" in kwargs, "missing exception object"
        assert "trace" in kwargs, "missing traceback"
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_class_skip(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_class_cancel(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_class_complete(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta"
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_class_in_progress(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_class_ignore(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_before_group_failure(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_before_group_error(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_after_group_failure(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"

    def on_after_group_error(self, **kwargs):
        assert "properties" in kwargs, "missing properties"
        assert "suite_meta" in kwargs["properties"], "missing suite meta"
        assert "jm" in kwargs["properties"], "missing test junkie meta: {}".format(kwargs)
        assert "jso" in kwargs["properties"]["jm"], "missing suite object in junkie meta"



import time

from test_junkie.decorators import Suite, test


@Suite()
class ParallelSuiteC:

    @test(priority=1, parameters=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
    def a(self, parameter):
        time.sleep(1)

    @test(priority=3, parameters=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
    def b(self, parameter):
        time.sleep(1)

    @test(priority=1)
    def c(self):
        time.sleep(1)

    @test(priority=2)
    def d(self):
        pass

    @test(priority=2)
    def e(self):
        time.sleep(1)

    @test(priority=1)
    def f(self):
        pass

    @test(priority=1)
    def g(self):
        time.sleep(1)

    @test(priority=1)
    def h(self):
        pass

    @test(priority=1)
    def i(self):
        time.sleep(1)

    @test(priority=1)
    def j(self):
        pass

    @test(priority=2)
    def k(self):
        time.sleep(1)



import time

from test_junkie.decorators import Suite, beforeClass, afterClass, test, afterTest, beforeTest
from tests.junkie_suites.TestListener import TestListener


@Suite(listener=TestListener)
class CancelSuite:

    @beforeClass()
    def before_class(self):
        time.sleep(2)

    @afterClass()
    def after_class(self):
        pass

    @beforeTest()
    def before_test(self):
        pass

    @afterTest()
    def after_test(self):
        pass

    @test()
    def pass_1(self):
        pass

    @test()
    def pass_2(self):
        pass



from test_junkie.decorators import Suite, test, afterTest, beforeTest
from test_junkie.rules import Rules
from tests.junkie_suites.TestListener import TestListener


@Suite(listener=TestListener, parameters=[1, 2], retry=2)
class ExecutionSequence1:

    @beforeTest()
    def before_test(self):
        assert True is False

    @afterTest()
    def after_test(self):
        pass

    @test()
    def test_1(self):
        pass

    @test()
    def test_2(self):
        pass

    @test(parameters=[1, 2], retry=2)
    def test_3(self, suite_parameter, parameter):
        pass


@Suite(listener=TestListener, parameters=[1, 2], retry=2)
class ExecutionSequence2:

    @beforeTest()
    def before_test(self):
        pass

    @afterTest()
    def after_test(self):
        assert True is False

    @test()
    def test_1(self):
        pass

    @test()
    def test_2(self):
        pass

    @test(parameters=[1, 2], retry=2)
    def test_3(self, suite_parameter, parameter):
        pass


@Suite(listener=TestListener, parameters=[1, 2], retry=2)
class ExecutionSequence3:

    @beforeTest()
    def before_test(self):
        pass

    @afterTest()
    def after_test(self):
        pass

    @test()
    def test_1(self):
        assert True is False

    @test()
    def test_2(self):
        pass

    @test(parameters=[1, 2], retry=2)
    def test_3(self, suite_parameter, parameter):
        pass


@Suite(listener=TestListener, parameters=[1, 2], rules=Rules)
class ExecutionSequence4:

    @beforeTest()
    def before_test(self):
        assert True is False

    @afterTest()
    def after_test(self):
        assert True is False

    @test(skip_after_test=True, skip_before_test=True,
          skip_after_test_rule=True, skip_before_test_rule=True)
    def test_1(self):
        pass

    @test(skip_after_test=True, skip_before_test=True,
          skip_after_test_rule=True, skip_before_test_rule=True)
    def test_2(self):
        pass

    @test(parameters=[1, 2],
          skip_after_test=True, skip_before_test=True,
          skip_after_test_rule=True, skip_before_test_rule=True)
    def test_3(self, suite_parameter, parameter):
        pass



from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener
from tests.junkie_suites.TestRules import TestRules


class Bound:

    def __init__(self):

        pass

    def get_params(self):

        return []


def get_params():
    return []


@Suite(listener=TestListener, parameters=Bound().get_params, retry=2)
class IgnoreSuiteBoundMethod:

    @test()
    def pass_1(self):
        pass


@Suite(listener=TestListener, parameters=get_params, retry=3)
class IgnoreSuiteFunction:

    @test()
    def pass_1(self):
        pass


@Suite(listener=TestListener, parameters=[], retry=2)
class IgnoreSuiteClassic:

    @test()
    def pass_1(self):
        pass


@Suite(listener=TestListener, parameters=get_params(), retry=2)
class IgnoreSuiteClassic2:

    @test()
    def pass_1(self):
        pass


@Suite(listener=TestListener, parameters=Bound().get_params(), retry=2)
class IgnoreSuiteClassic3:

    @test()
    def pass_1(self):
        pass


@Suite(listener=TestListener, retry=2, rules=TestRules)
class IgnoreSuiteBeforeGroupRule:

    @test()
    def ignore_1(self):
        pass

    @test()
    def ignore_2(self):
        pass


@Suite(listener=TestListener, retry=2, rules=TestRules)
class IgnoreSuiteBeforeGroupRule2:

    @test()
    def ignore_1(self):
        pass

    @test()
    def ignore_2(self):
        pass



from test_junkie.decorators import Suite, test


@Suite(owner="John Doe")
class SkipOwner:

    @test()
    def test_1(self):
        pass

    @test(owner="Jane Doe")
    def test_2(self):
        pass

    @test()
    def test_3(self):
        pass


@Suite(feature="Login")
class SkipFeature:

    @test()
    def test_1(self):
        pass

    @test()
    def test_2(self):
        pass

    @test()
    def test_3(self):
        pass


@Suite(feature="Login")
class SkipComponent:

    @test()
    def test_1(self):
        pass

    @test(component="Auth API")
    def test_2(self):
        pass

    @test()
    def test_3(self):
        pass


@Suite()
class SkipTests:

    @test()
    def test_1(self):
        pass

    @test()
    def test_2(self):
        pass

    @test()
    def test_3(self):
        pass



import pprint

from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.AfterTestAssertionSuite import AfterTestAssertionSuite
from tests.junkie_suites.AfterTestExceptionSuite import AfterTestExceptionSuite
from tests.junkie_suites.BeforeTestAssertionSuite import BeforeTestAssertionSuite
from tests.junkie_suites.BeforeTestExceptionSuite import BeforeTestExceptionSuite

runner = Runner([BeforeTestAssertionSuite])
runner.run()
results = runner.get_executed_suites()

pprint.pprint(results[0].metrics.get_metrics())
for test in results[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics():

    metrics = results[0].metrics.get_metrics()
    pprint.pprint(metrics)
    QualityManager.check_class_metrics(metrics,
                                       expected_retry_count=2,
                                       expected_status="fail",
                                       expected_beforetest_exception_count=8,
                                       expected_beforetest_exception_object=AssertionError,
                                       expected_beforetest_performance_count=8)


def test_failure():
    tests = results[0].get_test_objects()
    for test in tests:
        metrics = test.metrics.get_metrics()["None"]["None"]
        QualityManager.check_test_metrics(metrics,
                                          expected_status="fail",
                                          expected_retry_count=4,
                                          expected_exception_count=4,
                                          expected_performance_count=4,
                                          expected_exception=AssertionError)


runner2 = Runner([BeforeTestExceptionSuite])
runner2.run()
results2 = runner2.get_executed_suites()


def test_class_metrics2():

    metrics = results2[0].metrics.get_metrics()
    pprint.pprint(metrics)
    QualityManager.check_class_metrics(metrics,
                                       expected_retry_count=2,
                                       expected_status="fail",
                                       expected_beforetest_exception_count=8,
                                       expected_beforetest_exception_object=Exception,
                                       expected_beforetest_performance_count=8)


def test_failure2():
    tests = results2[0].get_test_objects()
    for test in tests:
        metrics = test.metrics.get_metrics()["None"]["None"]
        QualityManager.check_test_metrics(metrics,
                                          expected_status="error",
                                          expected_retry_count=4,
                                          expected_exception_count=4,
                                          expected_performance_count=4,
                                          expected_exception=Exception)


runner3 = Runner([AfterTestAssertionSuite])
runner3.run()
results3 = runner3.get_executed_suites()


def test_class_metrics3():

    metrics = results3[0].metrics.get_metrics()
    pprint.pprint(metrics)
    QualityManager.check_class_metrics(metrics,
                                       expected_retry_count=2,
                                       expected_status="fail",
                                       expected_aftertest_exception_count=8,
                                       expected_aftertest_exception_object=AssertionError,
                                       expected_aftertest_performance_count=8)


def test_failure3():
    tests = results3[0].get_test_objects()
    for test in tests:
        metrics = test.metrics.get_metrics()["None"]["None"]
        pprint.pprint(metrics)
        QualityManager.check_test_metrics(metrics,
                                          expected_status="fail",
                                          expected_retry_count=4,
                                          expected_exception_count=4,
                                          expected_performance_count=4,
                                          expected_exception=AssertionError)


runner4 = Runner([AfterTestExceptionSuite])
runner4.run()
results4 = runner4.get_executed_suites()


def test_class_metrics4():

    metrics = results4[0].metrics.get_metrics()
    pprint.pprint(metrics)
    QualityManager.check_class_metrics(metrics,
                                       expected_retry_count=2,
                                       expected_status="fail",
                                       expected_aftertest_exception_count=8,
                                       expected_aftertest_exception_object=Exception,
                                       expected_aftertest_performance_count=8)


def test_failure4():
    tests = results4[0].get_test_objects()
    for test in tests:
        metrics = test.metrics.get_metrics()["None"]["None"]
        QualityManager.check_test_metrics(metrics,
                                          expected_status="error",
                                          expected_retry_count=4,
                                          expected_exception_count=4,
                                          expected_performance_count=4,
                                          expected_exception=Exception)



import copy
import pprint
import sys
from test_junkie.debugger import LogJunkie
LogJunkie.enable_logging(10)
from tests.cli.CliTestSuite import AuthApiSuite

sys.path.insert(1, "E:/Development/test_junkie/")
# from tests.junkie_suites.IgnoreSuite import IgnoreSuiteWrongDatatype
# from playground.suiteB import ShoppingCartSuite
# from playground.suiteC import AuthApiSuite
# from playground.suiteD import NewProductsSuite
from test_junkie.objects import Limiter

sys.path.insert(1, __file__.split("playground")[0])
from test_junkie.reporter.html_reporter import Reporter


# from playground.suiteA import LoginSuite
from test_junkie.runner import Runner
Limiter.ACTIVE = False
Limiter.SUITE_THROTTLING = 10
Limiter.TEST_THROTTLING = 1
runner = Runner([AuthApiSuite])

# aggregator = runner.run(suite_multithreading_limit=4, test_multithreading_limit=4, tests=["sso_hit_negative_auth_limit"])
# pprint.pprint(aggregator.get_report_by_suite())
# suite_objects = runner.get_executed_suites()
#
# for suite in suite_objects:
#     print("=============")
#     new_suite = copy.deepcopy(suite)
#     print(new_suite.get_class_name())
#     tests = new_suite.get_test_objects()
#     print(tests)
#     print(new_suite.metrics)
#     print(new_suite.metrics.get_metrics())
#     print(new_suite.get_data_by_tags())
#     for test in tests:
#         new_test = copy.deepcopy(test)
#         print(new_test.metrics.get_metrics())

# tags = aggregator.get_report_by_tags()
# features = aggregator.get_report_by_features()
# totals = aggregator.get_basic_report()
# owners = aggregator.get_report_by_owner()
# reporter = Reporter("E:\Development\\test_junkie\\test_junkie\.resources", features, totals)
# pprint.pprint(reporter.get_dataset_per_feature())
# pprint.pprint(reporter.get_absolute_results())
# pprint.pprint(aggregator.get_report_by_owner())



import pprint

from test_junkie.runner import Runner
from tests.QualityManager import QualityManager
from tests.junkie_suites.IgnoreSuite import IgnoreSuiteBeforeGroupRule, IgnoreSuiteBeforeGroupRule2

runner = Runner([IgnoreSuiteBeforeGroupRule, IgnoreSuiteBeforeGroupRule2])
runner.run()

results = runner.get_executed_suites()
tests = results[0].get_test_objects()
pprint.pprint(results[0].metrics.get_metrics())
for test in tests:
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_before_group_exceptions():

    for suite in results:
        metrics = suite.metrics.get_metrics()
        QualityManager.check_class_metrics(metrics,
                                           expected_status="ignore",
                                           expected_retry_count=0)



from test_junkie.decorators import Suite, test, beforeClass, beforeTest, afterTest, afterClass
from test_junkie.meta import meta, Meta
from tests.junkie_suites.TestListener import TestListener
from tests.junkie_suites.TestRules import TestRules


@Suite(retry=2,
       listener=TestListener,
       rules=TestRules,
       meta=meta(name="Advanced Use Cases",
                 known_bugs=[]),
       parameters=[1, 2])
class AdvancedSuite:

    @beforeClass()
    def before_class(self, suite_parameter):
        pass

    @beforeTest()
    def before_test(self, suite_parameter):
        pass

    @afterTest()
    def after_test(self, suite_parameter):
        pass

    @afterClass()
    def after_class(self, suite_parameter):
        pass

    @test(retry=2, parameters=[10, 20], tags=["critical2", "v1"],
          meta=meta(name="No Retry", expected="Test must pass on 1st go, thus should not be retried"))
    def no_retry(self, parameter, suite_parameter):
        assert parameter is not None
        assert suite_parameter is not None
        Meta.update(self, parameter=parameter, suite_parameter=suite_parameter,
                    name="No Retry 1", expected="updated")
        Meta.get_meta(self, parameter=parameter, suite_parameter=suite_parameter)

    @test(retry=2, parameters=[10, 20], tags=["critical2", "v1"], parallalized_parameters=True,
          meta=meta(name="No Retry", expected="Test must pass on 1st go, thus should not be retried"))
    def no_retry2(self, parameter):
        assert parameter is not None
        Meta.update(self, parameter=parameter, name="No Retry 2", expected="updated")
        Meta.get_meta(self, parameter=parameter)

    @test(retry=2, tags=["critical2", "v1"],
          meta=meta(name="No Retry", expected="Test must pass on 1st go, thus should not be retried"))
    def no_retry3(self, suite_parameter):
        assert suite_parameter is not None
        Meta.update(self, suite_parameter=suite_parameter, name="No Retry 3", expected="updated")
        Meta.get_meta(self, suite_parameter=suite_parameter)

    @test(retry=2, tags=["critical2"])
    def retry(self):
        Meta.update(self, name="Retry", expected="Updated for Retry")
        Meta.get_meta(self)
        assert True is False, "Expected Assertion Error"

    @test(retry=2, tags=["trivial"])
    def retry2(self):
        Meta.update(self, name="new test name", expected="updated expectation")
        Meta.get_meta(self)
        assert True is False, "Expected Assertion Error 2"

    @test(retry=2, parameters=[10, 20], tags=["critical", "v2"],
          meta=meta(name="Retry 3", expected="Test must fail and be retried"))
    def retry3(self, parameter, suite_parameter):
        assert parameter is not None
        assert suite_parameter is not None
        Meta.update(self, parameter=parameter, suite_parameter=suite_parameter,
                    name="new test name", expected="updated expectation")
        Meta.get_meta(self, parameter=parameter, suite_parameter=suite_parameter)
        if suite_parameter == 1 and parameter == 10:
            raise Exception("On purpose")

    @test(retry=2, parameters=[10, 20], tags=["critical", "v2"],
          meta=meta(name="Retry 3", expected="Test must fail and be retried"))
    def retry4(self, parameter):
        assert parameter is not None
        Meta.update(self, parameter=parameter, name="new test name", expected="updated expectation")
        Meta.get_meta(self, parameter=parameter)
        if parameter == 10:
            raise Exception("On purpose")

    @test(retry=2, tags=["critical", "v2"],
          meta=meta(name="Retry 3", expected="Test must fail and be retried"))
    def retry5(self, suite_parameter):
        assert suite_parameter is not None
        Meta.update(self, suite_parameter=suite_parameter, name="new test name", expected="updated expectation")
        Meta.get_meta(self, suite_parameter=suite_parameter)
        if suite_parameter == 1:
            raise Exception("On purpose")

    @test(tags=["skip", "v2"])
    def skip(self):

        Meta.update(self, name="new test name", expected="updated expectation")
        Meta.get_meta(self)



import pprint
import sys


sys.path.insert(1, __file__.split("tests")[0])
from tests.QualityManager import QualityManager
from test_junkie.debugger import LogJunkie
LogJunkie.enable_logging(10)

from test_junkie.runner import Runner
from tests.junkie_suites.AdvancedSuite import AdvancedSuite

runner = Runner([AdvancedSuite])
runner.run(suite_multithreading=True, suite_multithreading_limit=5,
           test_multithreading=True, test_multithreading_limit=5,
           tag_config={"run_on_match_all": ["critical", "v2"],
                       "run_on_match_any": ["critical2"],
                       "skip_on_match_all": ["skip", "v2"],
                       "skip_on_match_any": ["trivial"]})
results = runner.get_executed_suites()
tests = results[0].get_test_objects()

pprint.pprint(results[0].metrics.get_metrics())
for test in tests:
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_stats():

    metrics = results[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="fail",
                                       expected_retry_count=2,

                                       expected_beforeclass_exception_count=4,
                                       expected_beforeclass_exception_object=None,
                                       expected_beforeclass_performance_count=4,

                                       expected_afterclass_exception_count=4,
                                       expected_afterclass_exception_object=None,
                                       expected_afterclass_performance_count=4,

                                       expected_beforetest_exception_count=29,
                                       expected_beforetest_exception_object=None,
                                       expected_beforetest_performance_count=29,

                                       expected_aftertest_exception_count=29,
                                       expected_aftertest_exception_object=None,
                                       expected_aftertest_performance_count=29)


def test_no_retry():

    tested = False
    for test in tests:
        if test.get_function_name() == "no_retry":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="success",
                                                      expected_param=param,
                                                      expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_no_retry2():

    tested = False
    for test in tests:
        if test.get_function_name() == "no_retry2":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="success",
                                                      expected_param=param,
                                                      expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_no_retry3():

    tested = False
    for test in tests:
        if test.get_function_name() == "no_retry3":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="success",
                                                      expected_param=param,
                                                      expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="fail",
                                                      expected_exception=AssertionError,
                                                      expected_retry_count=4,
                                                      expected_exception_count=4,
                                                      expected_performance_count=4,
                                                      expected_class_param=metrics["class_param"])
                    for i in metrics["exceptions"]:
                        assert str(i.with_traceback(None)) == "Expected Assertion Error"
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry2():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry2":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="skip",
                                                      expected_retry_count=4,
                                                      expected_exception_count=4,
                                                      expected_performance_count=4)
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry3():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry3":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    if metrics["param"] == 10 and metrics["class_param"] == 1:

                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="error",
                                                          expected_exception=Exception,
                                                          expected_retry_count=4,
                                                          expected_exception_count=4,
                                                          expected_performance_count=4,
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                        for i in metrics["exceptions"]:
                            assert str(i.with_traceback(None)) == "On purpose"
                    else:
                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="success",
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry4():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry4":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    if metrics["param"] == 10:

                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="error",
                                                          expected_exception=Exception,
                                                          expected_retry_count=4,
                                                          expected_exception_count=4,
                                                          expected_performance_count=4,
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                        for i in metrics["exceptions"]:
                            assert str(i.with_traceback(None)) == "On purpose"
                    else:
                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="success",
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry5():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry5":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    if metrics["class_param"] == 1:
                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="error",
                                                          expected_exception=Exception,
                                                          expected_retry_count=4,
                                                          expected_exception_count=4,
                                                          expected_performance_count=4,
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                        for i in metrics["exceptions"]:
                            assert str(i.with_traceback(None)) == "On purpose"
                    else:
                        QualityManager.check_test_metrics(metrics,
                                                          expected_status="success",
                                                          expected_param=param,
                                                          expected_class_param=metrics["class_param"])
                    tested = True
    if not tested:
        raise Exception("Test did not run")


def test_skip():

    tested = False
    for test in tests:
        if test.get_function_name() == "skip":
            for class_param, class_data in test.metrics.get_metrics().items():
                for param, metrics in class_data.items():
                    QualityManager.check_test_metrics(metrics,
                                                      expected_status="skip",
                                                      expected_retry_count=4,
                                                      expected_exception_count=4,
                                                      expected_performance_count=4)
                    tested = True
    if not tested:
        raise Exception("Test did not run")



from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener


@Suite(listener=TestListener)
class ErrorSuite3:

    @test(parameters={1, 2})
    def test_1(self, parameter):
        pass



from test_junkie.decorators import Suite, test


class A:

    def unbound(self):
        return [1, 2, 3]

    @classmethod
    def clsmethod(cls):
        return [1, 2, 3]

    @staticmethod
    def static():
        return [1, 2, 3]

    def unbound_wrong(self):
        return 1

    @classmethod
    def classmethod_wrong(cls):
        return "derp"

    @staticmethod
    def static_wrong():
        return {1: 2}


def func():
    return [1, 2, 3]


def func_wrong():
    return True


@Suite(parameters=A().unbound)
class ParametersSuite:

    def __init__(self):

        pass

    @test(parameters=A().unbound)
    def a(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=A().static)
    def b(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=A().clsmethod)
    def c(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=func)
    def d(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=A().unbound_wrong)
    def e(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=A().static_wrong)
    def f(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=A().classmethod_wrong)
    def g(self, parameter, suite_parameter):
        print(parameter, suite_parameter)

    @test(parameters=func_wrong)
    def h(self, parameter, suite_parameter):
        print(parameter, suite_parameter)



import sys
import time

sys.path.insert(1, __file__.split("tests")[0])

from test_junkie.runner import Runner
from tests.junkie_suites.Reporting import LoginSessions, Login, Dashboard


def test_reporting():
    f = __file__.replace("test_reporting.py", "test_{}".format(int(time.time())))
    html = "{}.html".format(f)
    xml = "{}.xml".format(f)
    runner = Runner([Login, LoginSessions, Dashboard],
                    monitor_resources=True, html_report=html, xml_report=xml)
    runner.run()

    suites = runner.get_executed_suites()
    for suite in suites:
        suite.get_average_performance_of_after_class()
        suite.get_average_performance_of_before_class()
        suite.get_average_performance_of_after_test()
        suite.get_average_performance_of_before_test()



import sys

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="test_junkie",
    version="0.7a2",
    author="Artur Spirin",
    author_email="as.no.replies@gmail.com",
    description="Modern Testing Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.test-junkie.com/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities"
    ],
    install_requires=["statistics", "psutil", "appdirs", "configparser",
                      "colorama", "coverage"],
    keywords=["automation", "testing", "tests", "test-runner"],
    entry_points={
          'console_scripts': [
              'tj2 = test_junkie.__main__:main'
              if sys.version_info[0] < 3 else 'tj3 = test_junkie.__main__:main',
              'tj = test_junkie.__main__:main'
          ]
    },
)



from test_junkie.decorators import Suite, test
from tests.junkie_suites.TestListener import TestListener


@Suite(listener=TestListener)
class ErrorSuite2:

    @test(parameters=[1, 2])
    def test_1(self):
        pass



import pprint

from test_junkie.debugger import LogJunkie
from tests.QualityManager import QualityManager

LogJunkie.enable_logging(10)
from test_junkie.runner import Runner
from tests.junkie_suites.Retry import Retries

runner = Runner([Retries])
runner.run()
results = runner.get_executed_suites()
tests = results[0].get_test_objects()
pprint.pprint(results[0].metrics.get_metrics())
for test in results[0].get_test_objects():
    print(test.get_function_name())
    pprint.pprint(test.metrics.get_metrics())


def test_class_metrics():

    metrics = results[0].metrics.get_metrics()
    QualityManager.check_class_metrics(metrics,
                                       expected_status="fail",
                                       expected_retry_count=3)


def test_retry_on_exception():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry_on_exception":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_retry_count=9,
                                              expected_performance_count=9,
                                              expected_exception_count=9,
                                              expected_status="error",
                                              expected_exception=Exception)
            tested = True
    if not tested:
        raise Exception("Test did not run")


def test_retry_on_assertion():

    tested = False
    for test in tests:
        if test.get_function_name() == "retry_on_assertion":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_retry_count=9,
                                              expected_performance_count=9,
                                              expected_exception_count=9,
                                              expected_status="fail",
                                              expected_exception=AssertionError)
            tested = True
    if not tested:
        raise Exception("Test did not run")


def test_no_retry_on_exception():

    tested = False
    for test in tests:
        if test.get_function_name() == "no_retry_on_exception":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_status="error",
                                              expected_exception=Exception)
            tested = True
    if not tested:
        raise Exception("Test did not run")


def test_no_retry_on_assertion():

    tested = False
    for test in tests:
        if test.get_function_name() == "no_retry_on_assertion":
            metrics = test.metrics.get_metrics()["None"]["None"]
            QualityManager.check_test_metrics(metrics,
                                              expected_status="fail",
                                              expected_exception=AssertionError)
            tested = True
    if not tested:
        raise Exception("Test did not run")



from test_junkie.decorators import Suite, beforeClass, afterClass, test


@Suite(skip=True)
class SkipSuite:

    @beforeClass()
    def before_class(self):
        raise Exception("Exception in before class")

    @afterClass()
    def after_class(self):
        raise Exception("Exception in after class")

    @test()
    def pass_1(self):
        pass

    @test()
    def pass_2(self):
        pass



class A:

    def do_skip(self):

        return True

    @classmethod
    def do_skip_classmethod(cls):
        return True

    @staticmethod
    def do_skip_static():
        return True

    def dont_skip(self):
        return False

    @classmethod
    def dont_skip_classmethod(cls):
        return False

    @staticmethod
    def dont_skip_static():
        return False


def do_skip():

    return True


def dont_skip():
    return False


@Suite(skip=A().dont_skip)
class SkipSuiteAdvanced:

    @test(skip=do_skip)
    def a(self):
        pass

    @test(skip=dont_skip)
    def b(self):
        pass

    @test(skip=A().do_skip)
    def c(self):
        pass

    @test(skip=A().dont_skip)
    def d(self):
        pass

    @test(skip=A().do_skip_static)
    def e(self):
        pass

    @test(skip=A().dont_skip_static)
    def f(self):
        pass

    @test(skip=A().do_skip_classmethod)
    def g(self):
        pass

    @test(skip=A().dont_skip_classmethod)
    def h(self):
        pass
